# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP Hizmet Borclanma Guncelle

Hitap'a personelin hizmet borclanma bilgilerinin eklenmesini yapar.

"""

from ulakbus.services.personel.hitap.hitap_guncelle import HITAPGuncelle


class HizmetBorclanmaGuncelle(HITAPGuncelle):
    """
    HITAP Guncelleme servisinden kalıtılmış Hizmet Borclanma Bilgisi Guncelleme servisi

    """
    HAS_CHANNEL = True

    def handle(self):
        """Servis çağrıldığında tetiklenen metod.

        Attributes:
            service_name (str): İlgili Hitap sorgu servisinin adı
            service_dict (dict): 'Request yoluyla kayıtlar,
                    HizmetBorclanmaUpdate servisinin alanlarıyla eşlenmektedir.
                    Filtreden geçecek tarih alanları listede tutulmaktadır.

        """

        self.service_name = 'HizmetBorclanmaUpdate'
        self.service_dict = {
            'fields': {
                'kayitNo': self.request.payload.get('kayit_no', ''),
                'tckn': self.request.payload.get('tckn', ''),
                'ad': self.request.payload.get('ad', ''),
                'soyad': self.request.payload.get('soyad', ''),
                'baslamaTarihi': self.request.payload.get('baslama_tarihi', ''),
                'bitisTarihi': self.request.payload.get('bitis_tarihi', ''),
                'gunSayisi': self.request.payload.get('gun_sayisi', ''),
                'odenenMiktar': self.request.payload.get('odenen_miktar', ''),
                'toplamTutar': self.request.payload.get('toplam_tutar', ''),
                'kanunKod': self.request.payload.get('kanun_kod', ''),
                'borcNevi': self.request.payload.get('borc_nevi', ''),
                'borclanmaTarihi': self.request.payload.get('borclanma_tarihi', ''),
                'odemeTarihi': self.request.payload.get('odeme_tarihi', ''),
                'derece': self.request.payload.get('derece', ''),
                'kademe': self.request.payload.get('kademe', ''),
                'ekgosterge': self.request.payload.get('ekgosterge', ''),
                'emekliSicil': self.request.payload.get('emekli_sicil', ''),
                'calistigiKurum': self.request.payload.get('calistigi_kurum', ''),
                'isyeriIl': self.request.payload.get('isyeri_il', ''),
                'isyeriIlce': self.request.payload.get('isyeri_ilce', ''),
                'kurumOnayTarihi': self.request.payload.get('kurum_onay_tarihi', '')
            },
            'date_filter': ['baslamaTarihi', 'bitisTarihi', 'borclanmaTarihi', 'kurumOnayTarihi'],
            'required_fields': ['tckn', 'kayitNo', 'ad', 'soyad', 'emekliSicil', 'derece', 'kademe',
                                'ekgosterge', 'baslamaTarihi', 'bitisTarihi', 'gunSayisi',
                                'kanunKod', 'borcNevi', 'toplamTutar', 'calistigiKurum', 'isyeriIl',
                                'isyeriIlce', 'kurumOnayTarihi']
        }
        super(HizmetBorclanmaGuncelle, self).handle()



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
#
# Yeni Personel Ekle WF adimlarini icerir.
from pyoko.exceptions import ObjectDoesNotExist
from zengine.views.crud import CrudView
from zengine.forms import JsonForm, fields
from zengine.lib.translation import gettext as _, gettext_lazy
from ulakbus.models.personel import Personel

"""
``button`` nesnelerinin stillendirilmesi için bu nesnelere ``style`` anahtarı ile css class'ları
gönderilebilir. Bu class'lar şunlar olabilir:
​
    - btn-default
    - btn-primary
    - btn-success
    - btn-info
    - btn warning
"""


# Views
class YeniPersonelEkle(CrudView):
    class Meta:
        model = 'Personel'

    def yeni_personel_form(self):
        """

        Yeni Personel eklenirken Tc Kimlik Numarası ekranı getirmek için kullanılan fonksiyon

        """
        _form = YeniPersonelTcknForm(current=self.current)
        self.form_out(_form)

    def mernis_kimlik_bilgileri_getir(self):
        tckn = self.current.input['form']['tckn']

        # Personelin daha önce bulunup bulunmadığını kontrol et
        try:
            Personel.objects.get(tckn=tckn)
            self.current.task_data['mernis_tamam'] = False
            self.current.task_data['hata_msg'] = _(u"Personel Daha Önce Kaydedilmiş")
        except ObjectDoesNotExist:
            from ulakbus.services.zato_wrapper import MernisKimlikBilgileriGetir, \
                MernisCuzdanBilgileriGetir

            # Kimlik bilgileri mernis servisi üzerinden çekilecek
            mernis_bilgileri = MernisKimlikBilgileriGetir(tckn=str(tckn))
            response = mernis_bilgileri.zato_request()
            self.current.task_data['mernis_tamam'] = True
            self.current.task_data['kimlik_bilgileri'] = response
            # Cüzdan bilgileri mernis servisi üzerinden çekilecek
            mernis_bilgileri = MernisCuzdanBilgileriGetir(tckn=str(tckn))
            response = mernis_bilgileri.zato_request()
            self.current.task_data['cuzdan_tamam'] = True
            self.current.task_data['cuzdan_bilgileri'] = response
            self.current.task_data['tckn'] = tckn

            self.current.set_message(title=_(u'%s TC no için Mernis servisi başlatıldı') % tckn,
                                     msg='', typ=1, url="/wftoken/%s" % self.current.token)

    def mernis_adres_bilgileri_getir(self, tckn=None):
        if not tckn:
            tckn = self.current.input['form']['tckn']
        # Adres bilgileri mernis servisi üzerinden çekilecek
        from ulakbus.services.zato_wrapper import KPSAdresBilgileriGetir
        mernis_bilgileri = KPSAdresBilgileriGetir(tckn=str(tckn))
        response = mernis_bilgileri.zato_request()

        self.current.task_data['adres_tamam'] = True
        self.current.task_data['adres_bilgileri'] = response

        return response

    def yeni_ekle_kontrol(self):
        _form = IletisimveEngelliDurumBilgileriForm(current=self.current)
        self.current.output['msgbox'] = {
            'type': 'info', "title": _(u'Personel Kayıt Uyarısı'),
            "msg": _(u"""
                      Lütfen Personel bilgilerinin doğruluğunu kontrol ediniz.
                      Kayıt işlemi geri döndürülemez.
                      """)
        }

        _form.ikamet_adresi = self.current.task_data['adres_bilgileri']['ikamet_adresi']
        _form.ikamet_il = self.current.task_data['adres_bilgileri']['ikamet_il']
        _form.ikamet_ilce = self.current.task_data['adres_bilgileri']['ikamet_ilce']

        _form.kaydet = fields.Button(_(u"Kaydet (Bu işlem geri alınamaz!)"), cmd="kaydet",
                                     btn='success')
        _form.iptal = fields.Button(_(u"İptal"), cmd="iptal", flow="iptal", form_validation=False)

        kimlik_bilgileri = _(u"""**Adı**: {ad}
                              **Soyad**: {soyad}
                              **Doğum Tarihi**: {dogum_tarihi}
                              """).format(**self.current.task_data['kimlik_bilgileri'])

        adres_bilgileri = _(u"""**İl**: {ikamet_il}
                             **İlçe**: {ikamet_ilce}
                             **Adres**: {ikamet_adresi}
                             """).format(**self.current.task_data['adres_bilgileri'])

        output = [{_(u'Kişi Bilgileri'): kimlik_bilgileri, _(u'Adres Bilgileri'): adres_bilgileri}]

        self.current.output['object'] = {
            "type": "table-multiRow",
            "title": _(u"Kişi Bilgileri"),
            "fields": output,
            "actions": False
        }
        self.form_out(_form)

    def kaydet(self):
        yeni_personel = Personel()
        yeni_personel.tckn = self.current.task_data['tckn']

        # Task data içinde gelen bilgiler birleştirilecek
        personel_data = {}
        personel_data.update(self.current.task_data['kimlik_bilgileri'])
        personel_data.update(self.current.task_data['cuzdan_bilgileri'])
        personel_data.update(self.current.task_data['adres_bilgileri'])

        for key in personel_data:
            setattr(yeni_personel, key, personel_data[key])

        yeni_personel.save()

        self.current.output['msgbox'] = {
            'type': 'success',
            "title": _(u'%(ad)s %(soyad)s Başarı İle Kaydedildi') % {
                'ad': yeni_personel.ad, 'soyad': yeni_personel.soyad,
            },
            "msg": _(u"""
                      Personel Başarı ile Kaydedildi, Personele atama yapabilir veya daha sonra
                      atama yapmak için "İşlemi Bitir" Butonuna tıklayabilirsiniz
                      """)
        }

        _form = JsonForm(current=self.current)

        # todo: bu buton ilgili personel secili olarak yeni bir wf baslatacak
        _form.geri = fields.Button(_(u"Atama Yap"), style="btn-success", cmd="atama_yap")

        _form.iptal = fields.Button(_(u"İşlemi Bitir"), cmd="bitir")
        self.form_out(_form)

        self.current.task_data['personel_id'] = yeni_personel.key

    def hata_goster(self):
        if self.current.task_data['hata_msg']:
            msg = self.current.task_data['hata_msg']
        else:
            msg = _(u"Bilinmeyen bir hata oluştu :( sebebini biliyorsanız bizede söyleyinde düzeltelim")
        self.current.output['msgbox'] = {
            'type': 'error', "title": _(u'İşlem Başarısız'),
            "msg": msg
        }

        _form = JsonForm(current=self.current)
        _form.geri = fields.Button(_(u"Yeni bir Tc Kimlik Numarası ile dene"), style="btn-success",
                                   cmd="tekrar")
        _form.iptal = fields.Button(_(u"İptal"), cmd="iptal")
        self.form_out(_form)


# Formlar
class YeniPersonelTcknForm(JsonForm):
    class Meta:
        title = gettext_lazy(u'Yeni Personel Ekle')
        help_text = gettext_lazy(u"Kimlik Numarası ile MERNIS ve HITAP bilgileri getir.")

    tckn = fields.String(gettext_lazy(u"TC No"))
    cmd = fields.String(gettext_lazy(u"Bilgileri Getir"), type="button")


class IletisimveEngelliDurumBilgileriForm(JsonForm):
    class Meta:
        title = gettext_lazy(u'Bilgi Kontrol Ekranı')
        help_text = gettext_lazy(u"Yeni Personelin Iletisim Bilgilerini Duzenle.")
        grouping = [
            {
                "layout": "6",
                "groups": [
                    {
                        "group_title": gettext_lazy(u"Adres Bilgileri"),
                        "items": ['ikamet_adresi', 'ikamet_il', 'ikamet_ilce',
                                  'adres_2', 'adres_2_posta_kodu'],
                        "collapse": True,
                    }
                ]
            },
            {
                "layout": "6",
                "groups": [
                    {
                        "group_title": gettext_lazy(u"İletişim Bilgileri"),
                        "items": ['oda_tel_no', 'cep_telefonu', 'e_posta', 'e_posta_2',
                                  'e_posta_3', 'web_sitesi', 'notlar'],
                        "collapse": True,
                    }
                ]
            },
            {
                "layout": "12",
                "groups": [
                    {
                        "group_title": gettext_lazy(u"Engel Bilgileri"),
                        "items": ['engelli_durumu', 'engel_grubu', 'engel_derecesi', 'engel_orani']
                    }
                ]
            }
        ]

    ikamet_adresi = fields.Text(gettext_lazy(u"İkamet Adresi"))
    ikamet_il = fields.String(gettext_lazy(u"İkamet Il"))
    ikamet_ilce = fields.String(gettext_lazy(u"İkamet Ilce"))
    adres_2 = fields.String(gettext_lazy(u"Adres 2"))
    adres_2_posta_kodu = fields.String(gettext_lazy(u"Adres 2 Posta Kodu"))
    oda_tel_no = fields.String(gettext_lazy(u"Oda Telefon Numarası"))
    cep_telefonu = fields.String(gettext_lazy(u"Cep Telefonu"))
    e_posta = fields.String(gettext_lazy(u"Kurum E-Posta(Yeni Personel için boş bırakınız)"), required=False)
    e_posta_2 = fields.String(gettext_lazy(u"E-Posta 2"))
    e_posta_3 = fields.String(gettext_lazy(u"E-Posta 3"))
    web_sitesi = fields.String(gettext_lazy(u"Web Sitesi"))
    notlar = fields.Text(gettext_lazy(u"Not"))
    engelli_durumu = fields.String(gettext_lazy(u"Engellilik"))
    engel_grubu = fields.String(gettext_lazy(u"Engel Grubu"))
    engel_derecesi = fields.String(gettext_lazy(u"Engel Derecesi"))
    engel_orani = fields.Integer(gettext_lazy(u"Engellilik Orani"))



# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

from pyoko import ListNode
from zengine.views.crud import CrudView
from zengine.forms import fields
from zengine.forms import JsonForm
from ulakbus.models.ogrenci import Okutman, Donem, Takvim
from ulakbus.lib.personel import personel_izin_gunlerini_getir
from ulakbus.models.ders_sinav_programi import DersEtkinligi
from datetime import datetime, date
from collections import OrderedDict
from ulakbus.lib.date_time_helper import AYLAR, ay_listele, resmi_tatil_gunleri_getir
from ulakbus.lib.common import get_akademik_takvim
from ulakbus.lib.date_time_helper import yil_ve_aya_gore_ilk_ve_son_gun
from zengine.lib.translation import gettext as _, gettext_lazy

guncel_yil = datetime.now().year
guncel_ay = datetime.now().month

# Guncel donem ve 5 onceki yili tuple halinde YIL listesinde tutar.
yil_secenekleri = [(yil, yil) for yil in range(guncel_yil, guncel_yil - 6, -1)]


class TarihForm(JsonForm):
    """
    Puantaj tablosu hazırlanırken ay ve yıl seçiminde
    kullanılan form.
    """

    yil_sec = fields.String(gettext_lazy(u'Yıl Seçiniz'), choices=yil_secenekleri,
                            default=guncel_yil)
    ay_sec = fields.String(gettext_lazy(u'Ay Seçiniz'), choices=ay_listele, default=guncel_ay)


class OkutmanListelemeForm(JsonForm):
    """
    Puantaj tablosu hazırlanacak olan okutmanların listesini
    gösteren form.
    """

    class Meta:
        inline_edit = ['secim']
        allow_add_listnode = False

    class OkutmanListesi(ListNode):
        secim = fields.Boolean(gettext_lazy(u"Seçim"), type="checkbox")
        okutman = fields.String(gettext_lazy(u'Okutman'))
        key = fields.String(hidden=True)


class PuantajFormu(JsonForm):
    class Meta:
        help_text = """
                     R: Resmi Tatil
                     İ: İzinli"""

    pdf_sec = fields.Button("Pdf Çıkar")


class DersUcretiHesaplama(CrudView):
    """
    Tüm okutmanların aylık olarak girdikleri derslerin
    ve ek derslerin saatlerini hesaplamaya yarayan iş akışı.
    Izin gunleri ve resmi tatil gunlerini de dikkate alir.
    """

    def tarih_sec(self):
        """
        Puantaj tablosunun hesaplanacağı ay ve yılı
        seçmeye yarar.
        """
        _form = TarihForm(current=self.current)
        _form.title = _(u'Puantaj Tablosu Hazırlamak İstediğiniz Yıl ve Ayı Seçiniz')
        _form.sec = fields.Button(_(u"İlerle"))
        self.form_out(_form)

    def donem_kontrol(self):
        """
        Seçilen ay ve yıla ait dönem olup olmadığını kontrol eder.
        """
        secilen_ay = self.input['form']['ay_sec']
        self.current.task_data["yil"] = self.input['form']['yil_sec']
        self.current.task_data["ay"] = secilen_ay

        # guncel olan ayın ismi getirilir.
        self.current.task_data["ay_isim"] = dict(AYLAR).get(secilen_ay)

        # TODO: aktif donem ve oncesine rastlayan donem varmi diye kontrol etmeliyiz.
        donem_list = Donem.takvim_ayina_rastlayan_donemler(self.current.task_data["yil"],
                                                           self.current.task_data["ay"])

        self.current.task_data['donem_sayi'] = True if donem_list else False

    def donem_secim_uyari(self):
        """
        Eğer seçilen ay ve yıla ait dönem bulunamamışsa uyarı verir.
        2016 Güz dönemindeyken 2016 Temmuz ayı (Yaz Dönemi) için bir sorgu
        istenirse daha o dönem açılmadığından hata verecektir.
        """

        _form = JsonForm(current=self.current, title=_(u"Dönem Bulunamadı"))
        _form.help_text = _(u"""Seçtiğiniz yıl ve aya ait dönem bulunamadı. Tarih
                          seçimine geri dönmek için Geri Dön butonuna, işlemi
                          iptal etmek için İptal butonuna basabilirsiniz.""")
        _form.geri_don = fields.Button(_(u"Geri Dön"), flow='tarih_sec')
        _form.iptal = fields.Button(_(u"İptal"), flow='islem_iptali_bilgilendir')
        self.form_out(_form)

    def islem_iptali_bilgilendir(self):

        """
        İşlem iptal edildiğinde kullanıcı iptal hakkında bilgilendirilir.
        """

        self.current.output['msgbox'] = {
            'type': 'info', "title": _(u'Durum Mesajı'),
            "msg": _(u'Puantaj cetveli görüntüleme işleminiz iptal edilmiştir.')
        }

    def okutman_sec(self):
        """
        Puantaj tablosunun hesaplanacağı okutmanların
        seçilmesine yarar. Bu okutmanlar işlem yapılan birime göre
        sorgulatılır. Bilgisayar Mühendisliği için yapılan sorguda
        sadece Bilgisayar Mühendisliği'nde bulunan okutmanlar ekrana
        gelecektir.
        """

        birim_no = self.current.role.unit.yoksis_no
        _form = OkutmanListelemeForm(current=self.current, title=_(u"Okutman Seçiniz"))

        # TODO: ilgili doneme ait okutmanlar listelenmeli.
        okutmanlar = [o for o in Okutman.objects for gorev_birimi in o.GorevBirimi
                      if gorev_birimi.yoksis_no == birim_no and gorev_birimi.donem.key
                      == Donem.guncel_donem(self.current).key]

        for okutman in okutmanlar:
            _form.OkutmanListesi(secim=True, okutman=okutman.__unicode__(), key=okutman.key)

        _form.sec = fields.Button(_(u"İlerle"))
        self.form_out(_form)

    def okutman_secim_kontrol(self):

        """
        İşlem yapılacak öğretim görevlisi seçilip seçilmediğini kontrol eder.
        """
        secilen_okutmanlar = []
        for okutman_secim in self.current.input['form']['OkutmanListesi']:
            if okutman_secim['secim']:
                secilen_okutmanlar.append(okutman_secim)

        self.current.task_data["secilen_okutmanlar"] = secilen_okutmanlar
        self.current.task_data["title"] = self.current.role.unit.name.upper()

        self.current.task_data["okutman_kontrol"] = True if secilen_okutmanlar else False

    def okutman_secim_uyari(self):

        """
        Hiç öğretim görevlisi seçilmediği durumda, uyarı verir.
        """

        _form = JsonForm(current=self.current, title=_(u"Öğretim Görevlisi Seçmelisiniz"))
        _form.help_text = _(u"""İşlem yapabilmek için en az bir öğretim görevlisi seçmelisiniz.
                                Öğretim görevlisi seçimine geri dönmek için Geri Dön butonuna,
                                işlemi iptal etmek için İptal butonuna basabilirsiniz.""")
        _form.geri_don = fields.Button(_(u"Geri Dön"), flow='okutman_sec')
        _form.iptal = fields.Button(_(u"İptal"), flow='islem_iptali_bilgilendir')
        self.form_out(_form)

    def ders_saati_turu_secme(self):
        """
        Ders Ücreti ya da Ek Ders Ücreti hesaplarından birini seçmeye yarar.
        """

        _form = JsonForm(current=self.current, title=_(u"Puantaj Tablosu Hesaplama Türü Seçiniz"))
        _form.ders = fields.Button(_(u"Ders Ücreti Hesapla"), cmd='ders_ucreti_hesapla')
        _form.ek_ders = fields.Button(_(u"Ek Ders Ücreti Hesapla"), cmd='ek_ders_ucreti_hesapla')
        self.form_out(_form)

    def ucret_hesapla(self):
        """
        Seçilen ay ve yıla göre, seçilen her bir öğretim elemanının
        resmi tatil ve izinleri dikkate alarak aylık ders saati planlamasını
        yapar
        """
        # Ders seçim türüne göre ayarlamalar yapılır.
        ek_ders, ders_turu = (True, 'Ek') if self.input['form']['ek_ders'] == 1 else (False, '')

        yil = self.current.task_data["yil"]  # girilen yil
        ay = self.current.task_data["ay"]  # girilen ayin orderi
        ay_isim = self.current.task_data["ay_isim"]  # ayin adi

        # verilen ayın son gunu bulunur 28,31 gibi
        ilk, son = yil_ve_aya_gore_ilk_ve_son_gun(yil, ay)
        ayin_son_gunu = son.day

        birim_unit = self.current.role.unit

        # Secilen ay hangi donemleri kapsiyor, kac donemi kapsıyorsa
        # o donemleri dondürür.
        donem_list = Donem.takvim_ayina_rastlayan_donemler(yil, ay)

        # Resmi tatillerin gununu (23, 12, 8) gibi döndürür.
        resmi_tatil_list = resmi_tatil_gunleri_getir(birim_unit, yil, ay)

        # Kapsadığı dönemlere göre ders baslangıc ve bitis tarihlerini
        # baz alarak kapsadığı her bir dönemin seçilen ayda hangi gün
        # aralıklarını kapsadığı bilgisini tuple olarak dondurur. Bir
        # dönemi kapsıyorsa bir tuple, iki dönemi kapsıyorsa iki tuple
        # döner. (1,12) (19,31) gibi
        tarih_araligi = ders_etkinligine_gore_tarih_araligi(donem_list, yil, ay, birim_unit)

        table_head = ['Öğretim Elemanı']
        table_head.extend([str(d) for d in range(1, ayin_son_gunu + 1)])
        table_head.append('Toplam')

        self.output['objects'] = [table_head]

        # Seçilen okutmanların bulunmaması demek,
        title = _(u"%(baslik)s %(yil)s-%(ay)s AYI %(ders)s DERS PUANTAJ TABLOSU") % {
            'baslik': self.current.task_data["title"],
            'yil': yil,
            'ay': ay_isim.upper(),
            'ders': ders_turu.upper(),
        }

        _form = PuantajFormu(title=title)
        self.form_out(_form)

        for secilen_okutman in self.current.task_data["secilen_okutmanlar"]:
            okutman = Okutman.objects.get(secilen_okutman['key'])

            # Seçilen okutmanın İzin ve Ücretsiz İzinlerini dikkate alarak, verilen ayda
            # hangi günler izinli olduğunu liste halinde döndürmeye yarayan method
            # [17,18,19] gibi
            personel_izin_list = personel_izin_gunlerini_getir(okutman, yil, ay)

            # Verilen döneme ve okutmana göre, haftada hangi gün kaç saat dersi
            # (seçilen seçeneğe göre ders veya ek ders) olduğunu gösteren
            # dictionarylerden oluşan liste, seçilen yıl ve ay bir dönemi kapsıyorsa
            # bir dict, iki dönemi kapsıyorsa iki dict döner.
            ders_etkinlik_list = doneme_gore_okutman_etkinlikleri(donem_list, okutman, ek_ders)

            # Bulunan tarih araliklarina, okutmanın aylık ders etkinliklerine göre
            # ayın hangi gününde dersi varsa kaç saat dersi olduğu bilgisi,
            # izinliyse İ, resmi tatilse R bilgisini bir dictionary e doldurur.
            okutman_aylik_plan, ders_saati = okutman_aylik_plani(donem_list, ders_etkinlik_list,
                                                                 resmi_tatil_list,
                                                                 personel_izin_list,
                                                                 tarih_araligi, yil, ay)

            # Okutmanın oluşturulan bilgilerinin ekrana basılacağı şeklinin oluşturulması
            okutman_bilgi_listesi = okutman_bilgileri_doldur(okutman, ayin_son_gunu,
                                                             okutman_aylik_plan, ders_saati)

            item = {
                "type": "table-multiRow",
                "fields": okutman_bilgi_listesi,
                "actions": False,
            }

            self.output['objects'].append(item)

    def tek_okutman_sec(self):
        okutman = self.current.user.personel.okutman
        self.current.task_data["secilen_okutmanlar"] = [{'key': okutman.key}]
        self.current.task_data["title"] = okutman.__unicode__().upper()


def ders_etkinligine_gore_tarih_araligi(donem_list, yil, ay, birim_unit):
    tarih_araligi = []

    # Hangi dönemin ders başlangıç ve bitiş tarihinin
    # dikkate alınacağını belirlemek için Akademik Takvim'de
    # bulunan etkinlik döneme göre seçilir.

    for donem in donem_list:

        # İçinde bulunulan dönemin ders başlangıç ve bitiş tarihlerini
        # bulmak için kullanılır.

        etkinlik = 66 if 'Güz' in donem.ad else 67 if 'Bahar' in donem.ad else 68

        # Kapsanan donemin ders baslangic tarihi
        akademik_takvim = get_akademik_takvim(birim_unit, donem.ogretim_yili)
        ders_bas_tarih = Takvim.objects.get(akademik_takvim=akademik_takvim,
                                            etkinlik=etkinlik).baslangic.date()

        # Kapsanan donemin ders bitis tarihi
        ders_bit_tarih = Takvim.objects.get(akademik_takvim=akademik_takvim,
                                            etkinlik=etkinlik).bitis.date()

        # Seçilen ayın ilk günü = 01.08.2016
        # Seçilen ayın son günü = 31.08.2016
        ay_baslangic, ay_bitis = yil_ve_aya_gore_ilk_ve_son_gun(yil, ay)

        # 08.08.2016 > 01.08.2016
        # baslangic_tarih = 08.08.2016
        # dönem başlangıcı ve ders başlangıcı seçilen ayın içindedir

        # 05.09.2016 > 01.08.2016
        # baslangic_tarih = 31.08.2016
        # dönem başlangıcı seçilen ayın içindedir ama ders başlangıcı diğer aya sarkmıştır

        # 15.07.2016 < 01.08.2016
        # baslangic_tarih = 01.08.2016
        # dönem başlangıcı ve ders başlangıcı seçilen aydan öncedir

        # 15.08.2016 < 31.08.2016
        # bitis_tarih = 15.08.2016
        # dönem bitişi ve ders bitişi seçilen ayın içindedir

        # 25.07.2016 < 31.08.2016
        # bitis_tarih = 01.08.2016
        # dönem bitişi seçilen ayın içindedir ama ders bitişi bir önceki aydır

        # 15.09.2016 > 31.08.2016
        # bitis_tarih = 31.08.2016
        # dönem bitişi ve ders bitişi seçilen aydan sonradır

        if not ders_bas_tarih > ay_baslangic:
            baslangic_tarih = ay_baslangic.day
        elif ders_bas_tarih.month == ay:
            baslangic_tarih = ders_bas_tarih.day
        else:
            baslangic_tarih = ay_bitis.day

        if not ders_bit_tarih < ay_bitis:
            bitis_tarih = ay_bitis.day
        elif ders_bit_tarih.month == ay:
            bitis_tarih = ders_bit_tarih.day
        else:
            bitis_tarih = ay_baslangic.day

        # Eğer tuple lardan birisi (1,1) ya da (31,31) gibiyse bu
        # seçilen ayın içinde bir döneme rast geldiğini fakat ders etkinliğine
        # rast gelmediğini gösterir, mesela Temmuz ayının seçildiğini varsayarsak
        # yaz dönem başlangıcı 25 Temmuz olsun, ama ders etkinliği 3 Ağustosa sarkmış
        # olsun. Bu durumda ders etkinlikleri seçilen aydan sonraki ayda olduğu için,
        # (31,31) tuple'ı döner, böylelikle hesaplama yapılırken döngünün dönmemesi
        # sağlanmış olur. Hesaplama yapılırken günlerin düzgün hesaplanabilmesi için,
        # tuple'ın ikinci elemanı bir arttırılır. Bunun içinde eğer tuple'ın iki elemanı
        # da eşitse hesaplama yapılırken bir arttırılacağından burada ikinci eleman
        # bir azaltılır.
        if baslangic_tarih == bitis_tarih:
            bitis_tarih -= 1
        tarih_araligi.append((baslangic_tarih, bitis_tarih))

    return tarih_araligi


def doneme_gore_okutman_etkinlikleri(donem_list, okutman, ek_ders):
    """
    Verilen dönemde okutmanın hangi günler kaç saat dersi olduğu
    bilgisini list of dict halinde döndürür. Günler 1-7 Pazt-Pazar
    şeklinde belirlenmiştir. Örnek: {1:4, 3:3, 4:2} gibi. Bu
    dict bize okutmanın Pazartesi 4 saat, Çarşamba günü 3 saatlik
    dersi olduğunu gösterir. Bir dönem varsa bir dict iki dönem varsa,
    iki dict dönecektir. [{1:4, 3:3, 4:2}, {2:5, 5:3}]

    Args:
        donem_list (list): donem objeleri listesi
        okutman (Okutman): okutman objesi
        ek_ders (Boolean): etkinlik ek_ders mi?

    Returns:
        (list): hafta gunu ve ders saati sozluk listesi

    """

    # kontrol ders ya da ek ders hesaplamasını gösterir.
    # Eğer kontrol True ise ders, False ise ek ders hesaplanır.

    ders_etkinlik_list = []
    for donem in donem_list:

        okutman_ders_dict = {}

        for etkinlik in DersEtkinligi.objects.filter(donem=donem, okutman=okutman, ek_ders=ek_ders):

            if not etkinlik.gun in okutman_ders_dict:
                okutman_ders_dict[etkinlik.gun] = etkinlik.sure
            else:
                okutman_ders_dict[etkinlik.gun] += etkinlik.sure

        ders_etkinlik_list.append(okutman_ders_dict)

    return ders_etkinlik_list


def okutman_aylik_plani(donem_list, ders_etkinlik_list, resmi_tatil_list, personel_izin_list,
                        tarih_araligi, yil, ay):
    # Tarih aralığı bir tuple olduğu için
    # her bir dönem için o dönemin tarih aralığı
    # dikkate alınır.
    # {1: 'R', 4: 3, 6: 1, 11: 3, 13: 1, 18: 3, 19: 'R', 20: 1, 25: 3, 27: 1}
    ders_saati = 0
    okutman_aylik_plan = {}
    for j, donem in enumerate(donem_list):

        for i in range(tarih_araligi[j][0], tarih_araligi[j][1] + 1):

            hafta_gun = date(yil, ay, i).isoweekday()
            # Eğer gün personel izin listesinde varsa
            if i in personel_izin_list:
                okutman_aylik_plan[i] = 'İ'

            # Eğer gün personel izin listesinde varsa
            elif i in resmi_tatil_list[j]:
                okutman_aylik_plan[i] = 'R'

            # Eğer o gün izinli ya da resmi tatil değilse
            # ve dersi varsa dictin gün keyine kaç saat olduğu
            # value'su eklenir ve ders saati arttırılır.
            elif hafta_gun in ders_etkinlik_list[j]:
                okutman_aylik_plan[i] = ders_etkinlik_list[j][hafta_gun]
                ders_saati += ders_etkinlik_list[j][hafta_gun]

    return okutman_aylik_plan, ders_saati


def okutman_bilgileri_doldur(okutman, ayin_son_gunu, okutman_aylik_plan, ders_saati):
    okutman_bilgi_listesi = OrderedDict({})

    okutman_bilgi_listesi['Öğretim Elemanı'] = okutman.__unicode__()

    for gun in range(1, ayin_son_gunu + 1):
        if gun in okutman_aylik_plan:
            okutman_bilgi_listesi[' ' + str(gun)] = str(okutman_aylik_plan[gun])
        else:
            okutman_bilgi_listesi[' ' + str(gun)] = ' '

    okutman_bilgi_listesi['Toplam'] = str(ders_saati)

    return okutman_bilgi_listesi



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
#
# Yeni Personel Ekle WF adimlarini icerir.
from collections import OrderedDict
from datetime import date

from pyoko.exceptions import ValidationError
from ulakbus.lib.view_helpers import prepare_choices_for_model
from ulakbus.models.hitap.hitap_sebep import HitapSebep
from ulakbus.models.hitap.hitap import HizmetKayitlari
from zengine.views.crud import CrudView
from zengine.forms import JsonForm, fields
from zengine.lib.translation import gettext as _, gettext_lazy
from ulakbus.models.personel import Personel, Atama, Kadro


class PersonelAtama(CrudView):
    """Personel Atama İş Akışı
    Personel atama işlemlerinin gerçekleştiği iş akışıdır.

    Personel atama iş akışı aşağıdaki adımlardan oluşur.

    Kadro Durumunu Kontrol Et:
    Açık kadroların olup olmadığını kontrol eder.

    Eksik Bilgileri Kontrol Et:
    Personelin eksik bilgileri olup olmadığını kontrol eder.

    Eksik Bilgi Form:
    Personele ait eksik bilgiler düzenlenir.

    Eksik Bilgi Kaydet:
    Personele ait eksik bilgiler kaydedilir.

    Atama Durumunu Kontrol Et:
    Personele daha önceden atama yapılıp yapılmadığını kontrol eder.

    Kadro Bilgileri Form:
    Personelin atama bilgileri doldurulur.

    Kadro Bilgileri Göster:
    Personelin kişi bilgilerini ve atama bilgilerini gösterir.

    Atama Kaydet:
    Personel atama bilgileri ve hizmet kayıtları bilgileri kaydedilir.

    Aatama Göster:
    Personelin atama ve kişi bilgileri gösterilir.

    Atama İptal:
    Atama işleminin iptal edildiğine dair bilgi mesajı ekrana basılır.

    Sonuç Bilgisi Göster:
    Personelin bilgilerinin  hitap bilgileri ile  eşlenip eşlenmediğinin sonucu ekrana basar.

    Hitap Bilgilerini Getir:
    Personelin hitap bilgilerini getirir.







    """

    class Meta:
        model = 'Personel'

    def kadro_durumunu_kontrol_et(self):
        """
        Açık kadroların olup olmadığını kontrol eder.

        """
        if not prepare_choices_for_model(Kadro, durum=2):
            self.current.task_data['kadro_bos'] = False
            self.current.output['msgbox'] = {
                'type': 'info', "title": _(u'Personel Atama Başarısız'),
                "msg": _(u"Kadrolar dolu olduğu için atama yapılamaz.")
            }
        else:
            self.current.task_data['kadro_bos'] = True

    def eksik_bilgileri_kontrol_et(self):
        """
        Personelin eksik bilgileri olup olmadığını kontrol eder.

        """
        try:
            personel = Personel.objects.get(self.input['id'])
        except KeyError:
            personel = Personel.objects.get(self.current.task_data['personel_id'])

        self.current.task_data['personel_id'] = personel.key
        self.current.task_data['personel'] = personel.clean_value()
        if personel.kurum_sicil_no_int:
            self.current.task_data['eksik_bilgi_yok'] = True
        else:
            self.current.task_data['eksik_bilgi_yok'] = False
            self.current.task_data['ilk_atama'] = True

    def eksik_bilgi_form(self):
        """
        Personele ait eksik bilgiler düzenlenir.

        """
        _form = EksikBilgiForm(current=self.current)
        personel = Personel.objects.get(self.current.task_data['personel_id'])
        _form.kurum_sicil_no_int = fields.Integer(_(u"Kurum Sicil No"), default=personel.kurum_sicil_no_int)
        _form.personel_turu = fields.Integer(_(u"Personel Tipi"), choices="personel_turu",
                                             default=personel.personel_turu)
        _form.unvan = fields.Integer(_(u"Personel Unvan"), choices="unvan_kod", required=False,
                                     default=personel.unvan)
        _form.emekli_sicil_no = fields.String(_(u"Emekli Sicil No"), default=personel.emekli_sicil_no)
        _form.hizmet_sinif = fields.Integer(_(u"Hizmet Sınıfı"), choices="hizmet_sinifi",
                                            default=personel.hizmet_sinifi)
        _form.statu = fields.Integer(_(u"Statü"), choices="personel_statu", default=personel.statu)
        _form.brans = fields.String(_(u"Branş"), default=personel.brans)
        _form.gorev_suresi_baslama = fields.Date(_(u"Görev Süresi Başlama"),
                                                 default=str(personel.gorev_suresi_baslama))
        _form.gorev_suresi_bitis = fields.Date(_(u"Görev Süresi Bitiş"),
                                               default=str(personel.gorev_suresi_bitis))
        _form.goreve_baslama_tarihi = fields.Date(_(u"Göreve Başlama Tarihi"),
                                                  default=str(personel.goreve_baslama_tarihi))
        # TODO: ulakbüs yavaşladığından dolayı choices slice edildi, typeahead olacak.
        _form.set_choices_of('baslama_sebep', choices=prepare_choices_for_model(HitapSebep, nevi=1)[0:10])
        # Date formatında mı olacak? yoksa 2 yıl 3 ay gibisinden mi?
        _form.mecburi_hizmet_suresi = fields.Date(_(u"Mecburi Hizmet Süresi"),
                                                  default=str(personel.mecburi_hizmet_suresi))
        _form.emekli_giris_tarihi = fields.Date(_(u"Emekliliğe Giriş Tarihi"),
                                                default=str(personel.emekli_giris_tarihi))

        self.form_out(_form)

    def eksik_bilgi_kaydet(self):
        """
        Personele ait eksik bilgiler kaydedilir.

        """
        from datetime import datetime

        def date_time_object(date_string):
            try:
                datetime.strptime(str(date_string), '%Y-%m-%d')
                return True
            except ValueError:
                return False

        personel = Personel.objects.get(self.current.task_data['personel_id'])
        for value in self.current.input['form']:
            if hasattr(personel, value):
                try:
                    if not date_time_object(self.current.input['form'][value]):
                        setattr(personel, value, self.current.input['form'][value])
                    else:
                        date_str = datetime.strptime(self.current.input['form'][value], '%Y-%m-%d').strftime("%d.%m.%Y")
                        setattr(personel, value, datetime.strptime(date_str, "%d.%m.%Y"))
                except ValidationError:
                    setattr(personel, value + "_id", str(self.current.input['form'][value]))
            else:
                continue

        personel.blocking_save()

    def atama_durumunu_kontrol_et(self):
        """
        Personele daha önceden atama yapılıp yapılmadığını kontrol eder.

        """
        personel = Personel.objects.get(self.current.task_data['personel_id'])
        if personel.atama.key:
            self.current.task_data['ilk_atama'] = False
            self.current.task_data['guncel_atama'] = personel.atama.key
        else:
            self.current.task_data['ilk_atama'] = True

    def kadro_bilgileri_form(self):
        """
        Personelin atama bilgileri doldurulur.

        """
        _form = KadroBilgiForm(current=self.current,
                               title="{ad} {soyad} adlı personelin atama bilgilerini doldurunuz.".format(
                                   **self.current.task_data['personel'])
                               )

        _form.set_choices_of('kadro', choices=prepare_choices_for_model(Kadro, durum=2))
        _form.set_choices_of('durum', choices=prepare_choices_for_model(HitapSebep, nevi=1)[0:10])
        self.form_out(_form)

    def kadro_bilgileri_goster(self):
        """
        Personelin kişi bilgilerini ve atama bilgilerini gösterir.
        """
        genel_bilgiler = _("""**Adı**: {ad}
                              **Soyadı**: {soyad}
                              **Personel Tipi**: {personel_turu}""").format(**self.current.task_data['personel'])
        atama = Atama.objects.get(self.current.task_data['guncel_atama'])
        atama_verileri = atama.clean_value()
        birim = Kadro.objects.get(atama_verileri["kadro_id"]).birim()
        atama_bilgileri = _("**Hizmet Sınıfı**: {hizmet_sinifi}\n**Birim**:{0}\n").format(birim, **atama_verileri)

        output = [{0: _(u'Genel Bilgiler'),
                   1: _(u'Atama Bilgileri')}]
        self.current.output['objects'] = output
        bilgiler = OrderedDict([(0, genel_bilgiler), (1, atama_bilgileri)])
        item = {
            "type": "table-multiRow",
            "fields": bilgiler,
            "actions": False
        }
        self.current.output['objects'].append(item)
        _form = JsonForm(current=self.current, title=_(u"Personel Atama Bilgileri"))
        _form.edit = fields.Button(_("Düzenle"), cmd="edit")
        _form.yeni_atama = fields.Button(_("Atama Yap"), cmd="yeni_atama", form_validation=False)
        self.form_out(_form)
        self.current.output["meta"]["allow_actions"] = False

    def atama_kaydet(self):
        """
        Personel atama bilgileri ve hizmet kayıtları bilgileri kaydedilir.

        """

        if not prepare_choices_for_model(Kadro, durum=2):
            self.current.task_data['atama_basarili'] = False
            self.current.output['msgbox'] = {
                'type': 'info', "title": _(u'Personel Atama Başarısız'),
                "msg": _(u"Kadrolar dolu olduğu için atama yapılamaz.")
            }
        else:
            self.current.task_data['atama_basarili'] = True
            atanacak_kadro = Kadro.objects.get(self.current.input['form']['kadro'])
            atama = Atama(personel_id=self.current.task_data['personel_id'])
            try:
                atama.kadro = atanacak_kadro
                atama.ibraz_tarihi = self.current.input['form']['ibraz_tarihi']
                atama.durum = HitapSebep.objects.get(self.current.input['form']['durum'])
                atama.nereden = self.current.input['form']['nereden']
                atama.atama_aciklama = self.current.input['form']['atama_aciklama']
                atama.goreve_baslama_tarihi = self.current.input['form']['goreve_baslama_tarihi']
                atama.goreve_baslama_aciklama = self.current.input['form'][
                    'goreve_baslama_aciklama']
                atama.blocking_save()
                self.current.task_data['atama_bilgileri'] = atama.clean_value()

                personel = Personel.objects.get(self.current.task_data['personel_id'])
                hk = HizmetKayitlari(personel=personel)
                hk.baslama_tarihi = date.today()
                hk.kurum_onay_tarihi = self.current.input['form']['kurum_onay_tarihi']
                hk.sync = 1  # TODO: Düzeltilecek, beta boyunca senkronize etmemesi için 1 yapıldı
                hk.blocking_save()
                self.current.task_data['h_k'] = hk.key
                print  "dskshskj"
            except Exception as e:
                # Herhangi bir hata oluşursa atama silinecek
                atama.delete()
                self.current.output['msgbox'] = {
                    'type': 'warning', "title": _(u'Hata Oluştu'),
                    "msg": _(u'%s') % e.message
                }

    def atama_goster(self):
        """
        Personelin atama ve kişi bilgileri gösterilir.

        """
        kisi_bilgileri = _(u"""**Adı**: {ad}
                              **Soyad**: {soyad}""").format(**self.current.task_data['personel'])
        personel = Personel.objects.get(self.current.task_data['personel_id'])
        atama_bilgileri = _(u"**Atama**: {atama}\n").format(atama=personel.atama)

        output = [{0: _(u'Kişi Bilgileri'),
                   1: _(u'Atama Bilgileri')}]
        self.current.output['objects'] = output
        bilgiler = OrderedDict([(0, kisi_bilgileri), (1, atama_bilgileri)])
        item = {
            "type": "table-multiRow",
            "title": _(u"Personel Ataması Başarı ile Tamamlandı"),
            "fields": bilgiler,
            "actions": False
        }
        self.current.output['objects'].append(item)

        _form = JsonForm(current=self.current, title=_(u"Kişi ve Atama Bilgileri"))

        _form.hitap = fields.Button(_(u"Hitap ile Eşleştir"), cmd="hitap_getir", btn='hitap')
        _form.bitir = fields.Button(_(u"İşlemi Bitir"), cmd="bitir", form_validation=False)
        self.form_out(_form)
        self.current.output["meta"]["allow_actions"] = False

    def atama_iptal(self):
        """
        Atama işleminin iptal edildiğine dair bilgi mesajı ekrana basılır.
        """
        self.current.output['msgbox'] = {
            'type': 'error', "title": _(u'Atama İptal Edildi'),
            "msg": _(u'Personel atama işlemi iptal edildi.'),
        }

    def sonuc_bilgisi_goster(self):
        """
        Personelin bilgilerinin  hitap bilgileri ile  eşlenip eşlenmediğinin sonucu ekrana basar.

        """
        hitap_sonuc = ''
        if 'hitap_tamam' in self.current.task_data:
            if self.current.task_data['hitap_tamam']:
                hitap_sonuc = _(u'Personel için hitap bilgileri Hitap sunucusu ile eşleştirildi.')
            else:
                hitap_sonuc = _(u"""
                    Personel için hitap bilgileri Hitap sunucusu ile eşleştirilemedi!!
                    Bu işlemi daha sonra tekrar başlatabilirsiniz.
                    """)

        self.current.output['msgbox'] = {
            'type': 'info', "title": _(u'Personel Atama Başarılı'),
            "msg": _(u'Atama İşlemi başarıyla gerçekleştirildi. ') + hitap_sonuc
        }

    def hitap_bilgi_getir(self):
        """
        Personelin hitap bilgilerini getirir.

        """
        personel = Personel.objects.get(self.current.task_data['personel_id'])
        from ulakbus.services.zato_wrapper import HitapHizmetCetveliSenkronizeEt
        hizmet_cetveli = HitapHizmetCetveliSenkronizeEt(tckn=str(personel.tckn))

        try:
            hizmet_cetveli.zato_request()
            self.current.task_data['hitap_tamam'] = True
        except:
            self.current.task_data['hitap_tamam'] = False


class EksikBilgiForm(JsonForm):
    """
    Personel Atama wf için form olarak kullanılır.

    """
    class Meta:
        title = _(u'Personelin Eksik Bilgileri')
        help_text = _(u"Atama Öncesi Personelin Eksik Bilgilerini Düzenlenleyiniz.")

        grouping = [
            {
                "layout": "6",
                "groups": [
                    {
                        "group_title": _(u"Genel Bilgiler"),
                        "items": ['kurum_sicil_no_int', 'personel_turu', 'unvan',
                                  'hizmet_sinif', 'statu', 'brans', 'emekli_sicil_no',
                                  'emekli_giris_tarihi'],
                        "collapse": True,
                    }
                ]
            },
            {
                "layout": "6",
                "groups": [
                    {
                        "group_title": _(u"Mecburi Hizmet Süresi"),
                        "items": ['mecburi_hizmet_suresi'],
                        "collapse": True,
                    }
                ]
            },
            {
                "layout": "6",
                "groups": [
                    {
                        "group_title": _(u"Görev Süresi"),
                        "items": ['gorev_suresi_baslama', 'gorev_suresi_bitis',
                                  'baslama_sebep', 'goreve_baslama_tarihi']
                    }
                ]
            }
        ]

    baslama_sebep = fields.Integer(_(u"Başlama Sebep"))
    kaydet = fields.Button(_(u"Kaydet"), cmd="kaydet", style="btn-success")
    iptal = fields.Button(_(u"İptal"), cmd="iptal", form_validation=False)


class KadroBilgiForm(JsonForm):
    """
    Personel Atama wf  için form olarak kullanılır.

    """
    class Meta:
        title = gettext_lazy(u'Atama Bilgileri')

    kadro = fields.String(_(u"Atanacak Kadro Seçiniz"))  # typeahead olacak
    ibraz_tarihi = fields.Date(_(u"İbraz Tarihi"))
    durum = fields.String(_(u"Durum"))  # typeahead olacak
    nereden = fields.Integer(_(u"Nereden"))
    atama_aciklama = fields.Text(_(u"Atama Açıklama"))
    goreve_baslama_tarihi = fields.Date(_(u"Göreve Başlama Tarihi"))
    goreve_baslama_aciklama = fields.String(_(u"Göreve Başlama Açıklama"))
    kurum_onay_tarihi = fields.Date(_(u"Kurum Onay Tarihi"))

    kaydet = fields.Button(_(u"Kaydet"), cmd="kaydet", style="btn-success")
    iptal = fields.Button(_(u"İptal"), cmd="iptal", form_validation=False)



# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

import time

from zato.server.service import Service
from ulakbus.models.zato import ZatoServiceChannel, ZatoServiceFile


class ServiceManagement(Service):

    @staticmethod
    def get_name():
        return "Service Management"

    def handle(self):
        ulakbus_service_files = ZatoServiceFile.objects.filter(deploy=False)
        if ulakbus_service_files:
            for ulakbus_service_file in ulakbus_service_files:

                self.invoke(
                        'zato.service.upload-package',
                        payload={"cluster_id": ulakbus_service_file.cluster_id,
                                 "payload_name": ulakbus_service_file.service_payload_name,
                                 "payload": ulakbus_service_file.service_payload})
                ulakbus_service_file.deploy = True
                ulakbus_service_file.save()
                time.sleep(0.3)

        ulakbus_channels = ZatoServiceChannel.objects.filter(deploy=False)

        if ulakbus_channels:
            for ulakbus_channel in ulakbus_channels:
                resp_service_name = self.invoke('zato.service.get-by-name',
                                                payload={
                                                    "cluster_id": 1,
                                                    "name": ulakbus_channel.service_name})
                time.sleep(0.3)
                ulakbus_channel.service_id = resp_service_name['zato_service_get_by_name_response']['id']

                resp_channel = self.invoke(
                    'zato.http-soap.create',
                    payload={"cluster_id": ulakbus_channel.cluster_id,
                             "name": ulakbus_channel.channel_name,
                             "is_active": ulakbus_channel.channel_is_active,
                             "connection": ulakbus_channel.channel_connection,
                             "transport": ulakbus_channel.channel_transport,
                             "is_internal": ulakbus_channel.channel_is_internal,
                             "url_path": ulakbus_channel.channel_url_path,
                             "data_format": ulakbus_channel.channel_data_format,
                             "service": ulakbus_channel.service_name})
                ulakbus_channel.channel_id = resp_channel['zato_http_soap_create_response']['id']

                ulakbus_channel.deploy = True
                ulakbus_channel.save()
                time.sleep(0.3)



# -*-  coding: utf-8 -*-
# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
#
"""Danışman Modülü

İlgili İş Akışlarına ait sınıf ve metotları içeren modüldür.

Dönem bazlı danışman atanmasını sağlayan iş akışını yönetir.

"""

from pyoko import ListNode
from pyoko.db.adapter.db_riak import BlockSave
from pyoko.exceptions import ObjectDoesNotExist
from zengine import forms
from zengine.forms import fields
from zengine.views.crud import CrudView
from ulakbus.models.ogrenci import Donem, DonemDanisman, Okutman
from ulakbus.models.auth import Unit, Role, AbstractRole
from ulakbus.views.ders.ders import prepare_choices_for_model
from zengine.lib.translation import gettext as _, gettext_lazy


class DonemDanismanForm(forms.JsonForm):
    """``DonemDanismanAtama`` sınıfı için form olarak kullanılacaktır.

    """

    ileri = fields.Button(gettext_lazy(u"İleri"))


class DonemDanismanListForm(forms.JsonForm):
    class Meta:
        inline_edit = ['secim']

    class Okutmanlar(ListNode):
        secim = fields.Boolean(gettext_lazy(u'Seçim'), type="checkbox")
        ad_soyad = fields.String(gettext_lazy(u'Ad Soyad'))
        key = fields.String(gettext_lazy(u'Key'), hidden=True)

    kaydet = fields.Button(gettext_lazy(u"Kaydet"))


class DonemDanismanAtama(CrudView):
    """Dönem Danışman Atama İş Akışı

    Dönem Danışman Atama, aşağıda tanımlı iş akışı adımlarını yürütür.

    - Öğretim Elemanlarını Seç
    - Kaydet
    - Kayıt Bilgisi Göster

     Bu iş akışında kullanılan metotlar şu şekildedir:

     Öğretim Elemanlarını Seç:
        Seçilen bölümdeki öğretim elemanları listelenir.

     Kaydet:
        Seçilen öğretim elemanları danışman olarak kaydeder.

     Kayıt Bilgisi Göster:
        Seçilen öğretim elemanları, dönem ve bölüm bilgilerinden oluşturulan kaydın mesajı
        gösterilir.
        Danışmanlara bilgilendirme mesajı gönderilir.
        Bu adımdan sonra iş akışı sona erer.

     Bu sınıf ``CrudView`` extend edilerek hazırlanmıştır. Temel model
     ``DonemDanisman`` modelidir. Meta.model bu amaçla kullanılmıştır.

     Adımlar arası geçiş manuel yürütülmektedir.

    """

    class Meta:
        model = "DonemDanisman"

    def danisman_sec(self):
        """
        Seçilen bölümdeki öğretim elemanları listelenir.

        """
        unit = self.current.role.unit
        self.current.task_data['unit_yoksis_no'] = unit.yoksis_no
        donem = Donem.guncel_donem(self.current)
        okutmanlar = [o for o in Okutman.objects for gorev_birimi in o.GorevBirimi if
                      gorev_birimi.yoksis_no == unit.yoksis_no and gorev_birimi.donem.key == donem.key]
        _form = DonemDanismanListForm(current=self, title=_(u"Okutman Seçiniz"))
        for okt in okutmanlar:
            try:
                DonemDanisman.objects.get(donem=donem, okutman=okt, bolum=unit)
                _form.Okutmanlar(secim=True, ad_soyad='%s %s' % (okt.ad, okt.soyad),
                                 key=okt.key)
            except ObjectDoesNotExist:
                _form.Okutmanlar(secim=False, ad_soyad='%s %s' % (okt.ad, okt.soyad),
                                 key=okt.key)

        self.form_out(_form)
        self.current.output["meta"]["allow_actions"] = False
        self.current.output["meta"]["allow_selection"] = False
        self.current.output["meta"]["allow_add_listnode"] = False

    def danisman_kaydet(self):
        """
        Seçilen öğretim elemanları danışman olarak kaydeder.

        """
        yoksis_no = self.current.task_data['unit_yoksis_no']
        unit = Unit.objects.get(yoksis_no=yoksis_no)
        donem = Donem.guncel_donem(self.current)
        danismanlar = self.current.input['form']['Okutmanlar']

        self.current.task_data['okutmanlar'] = []
        with BlockSave(DonemDanisman):
            for danisman in danismanlar:
                if danisman['secim']:
                    key = danisman['key']
                    okutman = Okutman.objects.get(key)
                    donem_danisman, is_new = DonemDanisman.objects.get_or_create(okutman=okutman,
                                                                                 donem=donem,
                                                                                 bolum=unit)
                    if is_new:
                        user = okutman.personel.user if okutman.personel.user else okutman.harici_okutman.user
                        abstract_role = AbstractRole.objects.get("DANISMAN")
                        unit = Unit.objects.get(yoksis_no=self.current.task_data['unit_yoksis_no'])
                        role = Role(user=user, unit=unit, abstract_role=abstract_role)
                        role.save()

                    self.current.task_data['okutmanlar'].append(okutman.key)

    def kayit_bilgisi_ver(self):
        """
        Seçilen öğretim elemanları, dönem ve bölüm bilgilerinden oluşturulan kaydın mesajı
        gösterilir.
        Danışmanlara bilgilendirme mesajı gönderilir.

        """
        yoksis_no = self.current.task_data['unit_yoksis_no']
        unit = Unit.objects.get(yoksis_no=yoksis_no)
        donem = Donem.guncel_donem(self.current)

        self.current.output['msgbox'] = {
            'type': 'info', "title": _(u'Danismanlar Kaydedildi'),
            "msg": _(
                u'%(donem)s dönemi için %(unit)s programına ait danışman listesi kaydedilmiştir') % {
                       'donem': donem, 'unit': unit}}

        title = _(u"Danışman Atama")
        message = _(u"%s dönemi için  danışman olarak atandınız.") % donem
        for okutman_key in self.current.task_data['okutmanlar']:
            okutman = Okutman.objects.get(okutman_key)
            user = okutman.personel.user if okutman.personel else okutman.harici_okutman.user
            abstract_role = AbstractRole.objects.get("OGRETIM_ELEMANI")
            try:
                role = Role.objects.get(user=user, abstract_role=abstract_role, unit=unit)
                role.send_notification(message=message, title=title, sender=self.current.user)
            except ObjectDoesNotExist:
                pass



# -*-  coding: utf-8 -*-
# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
#
from ulakbus.lib.views import UlakbusView
from zengine.forms import JsonForm
from zengine.forms import fields
from ulakbus.models import User
from zengine.lib.translation import gettext as _
from ulakbus.settings import DEMO_URL
from ulakbus.lib.common import aktivasyon_kodu_uret
from ulakbus.lib.common import ParolaSifirlama
from ulakbus.lib.common import kullanici_adi_var_mi


class ParolamiUnuttum(UlakbusView):
    """
    Kullanıcının parolasını sıfırlamak için bilgilerini girmesini ve
    birincil e-posta adreslerine doğrulama linki gönderilmesini sağlar.
    """

    class Meta:
        model = "User"

    def link_gecerliligi_kontrolu(self):
        """

        Eğer linkle başlatıldıysa, doğrulama linkinin
        geçerli olup olmadığı kontrol edilir.

        """
        self.current.task_data['kod'] = aktivasyon_kodu = self.current.input['model'].split('=')[1]

        self.current.task_data['kullanici_key'] = ParolaSifirlama(aktivasyon_kodu).get()

    def kullanici_adi_girisi(self):
        """

        Parolanın sıfırlanabilmesi için kullanıcıdan kullandığı kullanıcı adı
        istenir. Bu kullanıcı adına ait birincil e-posta adresine doğrulama
        linki gönderilecektir. Eğer gösterilecek bir mesaj varsa (yanlış
        kullanıcı adı girişi gibi) mesaj ekrana basılır.
        """
        if self.current.task_data.get('msg', None):
            self.mesaj_kutusu_goster(self.current.task_data.get('title', _(u"Hatalı Bilgi")))

        _form = JsonForm(current=self.current, title=_(u'Parola Sıfırlama'))
        _form.help_text = _(u"""Girdiğiniz kullanıcı adınıza kayıtlı birincil e-posta
                                adresinize parola sıfırlama linki gönderilecektir.""")
        _form.kullanici_adi = fields.String(_(u"Kullanıcı adınızı giriniz:"))
        _form.ilerle = fields.Button(_(u"Parola Sıfırlama Linki Gönder"))
        self.form_out(_form)

    def kullanici_adi_dogrulugu_kontrol(self):
        """
        Girilen kullanıcı adının veritabanında olup olmadığı kontrol edilir.
        Eğer hata yoksa doğrulama linki gönderilir, varsa hata mesajıyla
        birlikte bir önceki adıma gidilip hata mesajı gösterilir ve kullanıcıdan
        kullanıcı adını tekrar girmesi istenir.

        """

        self.current.task_data['kullanici_adi'] = kullanici_adi = self.input['form'][
            'kullanici_adi']
        self.current.task_data['dogruluk'] = kullanici_adi_var_mi(kullanici_adi)
        if not self.current.task_data['dogruluk']:
            self.current.task_data['msg'] = _(
                u"Böyle bir kullanıcı bulunmamaktadır. Lütfen tekrar deneyiniz.")

    def kullanici_bilgisi_cache_koy_e_posta_hazirla(self):
        """
        Doğrulama linki gönderilecek kullanıcının keyi,
        oluşturulan aktivasyon kodu ile cache'e kaydedilir.
        Gönderilecek e-postanın içeriği ve linki wf ismi ve aktivasyon kodu ile hazırlanır.
        """

        user = User.objects.get(username=self.current.task_data['kullanici_adi'])
        self.current.task_data['e_posta'] = user.e_mail
        self.current.task_data["aktivasyon"] = aktivasyon_kodu_uret()
        ParolaSifirlama(self.current.task_data["aktivasyon"]).set(user.key, 7200)

        self.current.task_data["message"] = "E-Posta adresinizi doğrulamak için aşağıdaki" \
                                            " linke tıklayınız:\n\n %s/#/%s/dogrulama=%s" % (
            DEMO_URL, self.current.task_data['wf_name'], self.current.task_data["aktivasyon"])

        self.current.task_data['subject'] = 'Ulakbüs Aktivasyon Maili'



# -*-  coding: utf-8 -*-
# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
#
from ulakbus.lib.views import UlakbusView
from zengine.forms import JsonForm
from zengine.forms import fields
from ulakbus.models import User
from ulakbus.lib.common import EPostaDogrulama
from zengine.lib import translation
import re
from zengine.lib.translation import gettext as _, gettext_lazy as __

aktivasyon_kalibi = re.compile('dogrulama=[a-z0-9]{40}')


class KullaniciForm(JsonForm):
    class Meta:
        exclude = ['password', 'superuser', 'harici_okutman', 'ogrenci', 'personel',
                   'last_login_role_key']


class EPostaForm(JsonForm):
    birincil_e_posta = fields.String(__(u"Birincil e-postanız"))


class ProfilGoruntule(UlakbusView):
    """
    Kullanıcıların profil sayfalarını görüntüleyebilmelerini,
    kullanıcı adı, şifre, e-posta, dil seçeneği gibi değişiklikleri
    yapabilmelerini sağlar.

    """

    class Meta:
        model = "User"

    def linkle_gelinmesi_kontrolu(self):
        """
        İş akışının doğrulama linkiyle mi yoksa menüden mi başlatıldığı
        kontrol edilir. Gönderilen doğrulama linki 'dogrulama=4jnjlam90a..'
        şeklindedir. Bu yapıya uyup uymadığı kontrol edilir. İlk try a
        girmez ise linkle gelmemiştir iş akışı menüden başlatılmıştır.

        """
        self.current.task_data['msg'] = None
        self.current.task_data['wf_name'] = self.current.workflow.name
        dogrulama = self.current.input.get('model', '')
        self.current.task_data['link'] = bool(re.match(aktivasyon_kalibi, dogrulama))

    def link_gecerliligi_kontrolu(self):
        """

        Eğer linkle başlatıldıysa, doğrulama linkinin
        geçerli olup olmadığı kontrol edilir.

        """
        self.current.task_data['kod'] = aktivasyon_kodu = self.current.input['model'].split('=')[1]
        self.current.task_data['e_posta'] = EPostaDogrulama(aktivasyon_kodu).get()

    def gecersiz_link_mesaji(self):
        """
        Bağlantı linki geçersiz olmuşsa, uyarı mesajı oluşturulur.

        """
        self.current.task_data['msg'] = _(u"""E-Postanızı onaylamanız için
            gönderilen link geçersiz olmuştur. Lütfen tekrar deneyiniz.""")
        self.current.task_data['title'] = _(u'Geçersiz İşlem')
        self.current.task_data['type'] = 'warning'

    def e_posta_degistir(self):
        """
        E_posta adresi doğrulanmışsa, e_posta değiştirme işlemi yapılır,
        değiştirilecek e_posta bilgisi Cache'den alınır, işlem yapıldıktan sonra
        Cache'den silinir.

        """

        kullanici = User.objects.get(self.current.user_id)
        kullanici.e_mail = self.current.task_data['e_posta']
        kullanici.save()
        EPostaDogrulama(self.current.task_data['kod']).delete()
        self.current.task_data['title'] = _(u'E-Posta Değişikliği')
        self.current.task_data['msg'] = _(
            u'E-Posta değiştirme işleminiz başarıyla gerçekleştirilmiştir.')
        self.current.task_data['type'] = 'info'

    def profil_sayfasi_goruntule(self):
        """
        Profil sayfasının ana görüntülenme ekranını oluşturur, eğer geçersiz link uyarısı,
        başarılı işlem, hatalı işlem gibi görüntülenecek mesaj varsa burada gösterilir.

        """
        if self.current.task_data.get('msg', None):
            self.mesaj_kutusu_goster(self.current.task_data['title'], self.current.task_data['type'])

        u = User.objects.get(self.current.user_id)
        _form = KullaniciForm(u, current=self.current,
                              title=_(u'%s %s Profil Sayfası') % (u.name, u.surname))
        _form.sifre_degistir = fields.Button(_(u"Parola Değiştir"), flow="parola_degistir")
        _form.k_adi_degistir = fields.Button(_(u"Kullanıcı Adı Değiştir"),
                                             flow="kullanici_adi_degistir")
        _form.e_posta_degistir = fields.Button(_(u"E-Posta Değiştir"), flow="e_posta_degistir")
        _form.kaydet = fields.Button(_(u"Kaydet"), flow="kaydet")
        self.form_out(_form)

    def degisiklik_sonrasi_islem(self):
        """
        Kullanıcı adı veya parola değişikliğinden sonra kullanıcıya iki seçenek sunulur.
        Çıkış yapması ya da çıkış yapmadan devam etmesi, bunu seçebileceği ekran gösterilir.

        """
        self.current.task_data['msg'] = _(u"""%s ya da çıkış yapmadan devam edebilirsiniz.
                            Eğer eski bilgilerinizin bilindiği şüphesine sahipseniz 'Çıkış Yap'
                            seçeneğini seçmenizi öneririz.""") % self.current.task_data[
            'islem_mesaji']
        self.mesaj_kutusu_goster('İşlem Bilgilendirme', 'info')

        _form = JsonForm(current=self.current, title=_(u"İşlem Seçeneği"))
        _form.cikis = fields.Button(_(u"Çıkış Yap"), flow='cikis_yap')
        _form.devam = fields.Button(_(u"Çıkış Yapmadan Devam Et"))
        self.form_out(_form)

    def dil_sayi_zaman_ayarlari_degistir(self):
        """
        Profil sayfasında dil seçenekleri, sayı ve zaman formatları kullanıcı tarafından
        değiştirilebilir. Bu değişiklikler kaydedilir ve bu bilgiler session da tutulduğu için
        session bilgisi sıfırlanır. Session'da bulamadığında bu bilgileri bir kereye mahsus
        olmak üzere modelden değişmiş halini alır ve en son halini sessiona koyar.

        """

        kullanici = self.current.user

        for k in translation.DEFAULT_PREFS.keys():
            self.current.session[k] = ''
            setattr(kullanici, k, self.input['form'][k])
        kullanici.blocking_save()
        self.current.output['cmd'] = 'reload'
        self.current.output['title'] = _(u'Dil, Sayı, Zaman Formatı Değişikliği')
        self.current.output['msg'] = _(u'Değişiklikleriniz başarıyla kaydedilmiştir. Ana sayfaya yönlendiriliyorsunuz.')


# -*-  coding: utf-8 -*-
"""Roller ile ilgili yardımcı class ve fonksiyonların yer aldığı dosyadır.

"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
from enum import Enum
from zengine.lib.translation import gettext_lazy as __


class AbsRole(Enum):
    """
    Abstract roller icin enumerated data. Veritabani keyleri ozellik adi ile eslesir.
    Kod icinde su sekilde kullanilabilir:

    ... code-block:: python
        hitap_kaydi = form_data["soyut_rol_id"] in [AbsRole.FAKULTE_DEKANI, AbsRole.REKTOR]

    kod icinde translation ile birlikte kullanilabilir.

    .. code-block:: python
        __("%s degistirildi..") % AbsRole.FAKULTE_YONETIM_KURULU_UYESI

    """

    FAKULTE_YONETIM_KURULU_UYESI = __(u"Fakülte Yönetim Kurulu Üyesi")
    FAKULTE_YONETIM_KURULU_BASKANI_DEKAN = __(u"Fakülte Yönetim Kurulu Başkanı (Dekan)")
    FAKULTE_DEKAN_SEKRETERI = __(u"Fakülte Dekan Sekreteri")
    FAKULTE_DEKANI = __(u"Fakülte Dekanı")
    FAKULTE_OGRENCI_ISLERI_SEFI = __(u"Fakülte Öğrenci İşleri Şefi")
    FAKULTE_KURULU_UYESI = __(u"Fakülte Kurulu Üyesi")
    FAKULTE_SEKRETERI = __(u"Fakülte Sekreteri")
    FAKULTE_ETIK_KURULU_BASKANI = __(u"Fakülte Etik Kurulu Başkanı")
    FAKULTE_ETIK_KURULU_UYESI = __(u"Fakülte Etik Kurulu Üyesi")
    FAKULTE_OGRENCI_ISLERI_PERSONELI = __(u"Fakülte Öğrenci İşleri Personeli")
    FAKULTE_DEKAN_YARDIMCISI = __(u"Fakülte Dekan Yardımcısı")
    FAKULTE_KURULU_BASKANI = __(u"Fakülte Kurulu Başkanı (Dekan)")
    ENSTITU_MUDURU = __(u"Enstitü Müdürü")
    ENSTITU_YONETIM_KURULU_BASKANI = __(u"Enstitü Yönetim Kurulu Başkanı")
    ENSTITU_OGRENCI_ISLERI_PERSONELI = __(u"Enstitü Öğrenci İşleri Personeli")
    ENSTITU_YONETIM_KURULU_UYESI = __(u"Enstitü Yönetim Kurulu Üyesi")
    ENSTITU_SEKRETERI = __(u"Enstitü Sekreteri")
    ENSTITU_OGRENCI_ISLERI_SEFI = __(u"Enstitü Öğrenci İşleri Şefi")
    ENSTITU_MUDUR_YARDIMCISI = __(u"Enstitü Müdür Yardımcısı")
    ENSTITU_MUHASEBE_ISLERI_PERSONELI = __(u"Enstitü Muhasebe İşleri Personeli")
    ENSTITU_KURULU_UYESI = __(u"Enstitü Kurulu Üyesi")
    ENSTITU_MUHASEBE_ISLERI_SEFI = __(u"Enstitü Muhasebe İşleri Şefi")
    ENSTITU_KURULU_BASKANI = __(u"Enstitü Kurulu Başkanı")
    YUKSELOKUL_KURULU_BASKANI = __(u"Yükselokul Kurulu Başkanı")
    YUKSELOKUL_YONETIM_KURULU_BASKANI = __(u"Yükselokul Yönetim Kurulu Başkanı")
    YUKSELOKUL_BIRIM_KOORDINATORU = __(u"Yükselokul Birim Koordinatörü")
    YUKSELOKUL_YONETIM_KURULU_UYESI = __(u"Yükselokul Yönetim Kurulu Üyesi")
    YUKSELOKUL_MUDURU = __(u"Yükselokul Müdürü")
    YUKSELOKUL_MUDUR_YARDIMCISI = __(u"Yükselokul Müdür Yardımcısı")
    YUKSELOKUL_SEKRETERI = __(u"Yükselokul Sekreteri")
    YUKSELOKUL_OGRENCI_ISLERI_SEFI = __(u"Yükselokul Öğrenci İşleri Şefi")
    YUKSELOKUL_KURULU_UYESI = __(u"Yükselokul Kurulu Üyesi")
    YUKSELOKUL_MUHASEBE_ISLERI_SEFI = __(u"Yükselokul Muhasebe İşleri Şefi")
    YUKSELOKUL_MUHASEBE_ISLERI_PERSONELI = __(u"Yükselokul Muhasebe İşleri Personeli")
    YUKSELOKUL_OGRENCI_ISLERI_PERSONELI = __(u"Yükselokul Öğrenci İşleri Personeli")
    TIP_FAKULTESI_EGITIM_KOMISYONU_UYESI = __(u"Tıp Fakültesi Eğitim Komisyonu Üyesi")
    TIP_FAKULTESI_BAS_KOORDINATOR_YARDIMCISI = __(u"Tıp Fakültesi Baş Koordinatör Yardımcısı")
    TIP_FAKULTESI_EGITIM_KOMISYONU_BASKANI = __(u"Tıp Fakültesi Eğitim Komisyonu Başkanı")
    TIP_FAKULTESI_DONEM_KOORDINATORU = __(u"Tıp Fakültesi Dönem Koordinatörü")
    TIP_FAKULTESI_BAS_KOORDINATORU = __(u"Tıp Fakültesi Baş Koordinatörü")
    ON_LISANS_OGRENCISI_AKTIF = __(u"Ön Lisans Programı Öğrencisi - Aktif")
    ON_LISANS_OGRENCISI_KAYIT_SILINMIS = __(u"Ön Lisans Programı Öğrencisi - Kayıt Silinmiş")
    ON_LISANS_OGRENCISI_KAYIT_DONDURMUS = __(u"Ön Lisans Programı Öğrencisi - Kayıt Dondurmuş")
    LISANS_OGRENCISI_AKTIF = __(u"Lisans Programı Öğrencisi - Aktif")
    LISANS_OGRENCISI_KAYIT_SILINMIS = __(u"Lisans Programı Öğrencisi - Kayıt Silinmiş")
    LISANS_OGRENCISI_KAYIT_DONDURMUS = __(u"Lisans Programı Öğrencisi - Kayıt Dondurmuş")
    YUKSEK_LISANS_OGRENCISI_AKTIF = __(u"Yüksek Lisans Programı Öğrencisi - Aktif")
    YUKSEK_LISANS_OGRENCISI_KAYIT_SILINMIS = __(u"Yüksek Lisans Programı Öğrencisi - Kayıt Silinmiş")
    YUKSEK_LISANS_OGRENCISI_KAYIT_DONDURMUS = __(u"Yüksek Lisans Programı Öğrencisi - Kayıt Dondurmuş")
    DOKTORA_OGRENCISI_AKTIF = __(u"Doktora Programı Öğrencisi - Aktif")
    DOKTORA_OGRENCISI_KAYIT_SILINMIS = __(u"Doktora Programı Öğrencisi - Kayıt Silinmiş")
    DOKTORA_OGRENCISI_KAYIT_DONDURMUS = __(u"Doktora Programı Öğrencisi - Kayıt Dondurmuş")
    OGRETIM_ELEMANI = __(u"Öğretim Elemanı")
    ANA_BILIM_DALI_UYESI = __(u"Ana Bilim Dalı Üyesi")
    ANA_BILIM_DALI_BASKANI = __(u"Ana Bilim Dalı Başkanı")
    BILIM_DALI_UYESI = __(u"Bilim Dalı Üyesi")
    BILIM_DALI_BASKANI = __(u"Bilim Dalı Başkanı")
    BOLUM_BASKANI = __(u"Bölüm Başkanı")
    BOLUM_KURULU_BASKANI = __(u"Bölüm Kurulu Başkanı")
    BOLUM_KURULU_UYESI = __(u"Bölüm Kurulu Üyesi")
    BOLUM_SEKRETERI = __(u"Bölüm Sekreteri")
    DAIRE_BASKANI = __(u"Daire Başkanı")
    DAIRE_SUBE_SEFI = __(u"Şube Şefi")
    DAIRE_SUBE_MUDURU = __(u"Daire Şube Müdürü")
    DAIRE_PERSONELI = __(u"Daire Personeli")
    REKTOR = __(u"Rektör")
    DANISMAN = __(u"Danışman")
    DERS_PROGRAMI_KOORDINATORU = __(u"Ders Programı Koordinatörü")
    BASEABSROLE = __(u"BaseAbsRole")
    GENEL_SEKRETER = __(u"Genel Sekreter")
    MUTEMET = __(u"Mutemet")


# -*-  coding: utf-8 -*-
# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""Personellerin  Kimlik ve İletişim  Bilgileri  İş Akışına ait
sınıf ve metotları içeren modüldür.

Kimlik ve İletişim Bilgileri iş akışının yürütülmesini sağlar.

"""

from zengine.forms import JsonForm
from zengine.forms import fields
from zengine.lib.translation import gettext as _, gettext_lazy

from zengine.views.crud import CrudView
from ulakbus.services.zato_wrapper import MernisKimlikBilgileriGetir
from ulakbus.services.zato_wrapper import KPSAdresBilgileriGetir


class KimlikBilgileriForm(JsonForm):
    """
    ``KimlikIletişim`` sınıfı için form olarak kullanılacaktır. Form,
    include listesinde, aşağıda tanımlı alanlara sahiptir.

    """

    class Meta:
        include = ['tckn', 'ad', 'soyad', 'cinsiyet', 'uyruk', 'medeni_hali', 'cuzdan_seri',
                   'cuzdan_seri_no', 'baba_adi', 'ana_adi', 'dogum_tarihi', 'dogum_tarihi',
                   'dogum_yeri', 'medeni_hali', 'kayitli_oldugu_il', 'kayitli_oldugu_ilce',
                   'kayitli_oldugu_mahalle_koy',
                   'kayitli_oldugu_cilt_no', 'kayitli_oldugu_aile_sira_no',
                   'kayitli_oldugu_sira_no',
                   'kimlik_cuzdani_verildigi_yer', 'kimlik_cuzdani_verilis_nedeni',
                   'kimlik_cuzdani_kayit_no',
                   'kimlik_cuzdani_verilis_tarihi']

    kaydet = fields.Button(gettext_lazy(u"Kaydet"), cmd="save")
    mernis = fields.Button(gettext_lazy(u"Mernis'ten Kimlik Bilgileri Getir"), cmd="mernis_kimlik_sorgula")


class IletisimBilgileriForm(JsonForm):
    """
    ``KimlikIletisim`` sınıfı için form olarak kullanılacaktır. Form,
    include listesinde, aşağıda tanımlı alanlara sahiptir.

    """

    class Meta:
        include = ['ikamet_adresi', 'ikamet_il', 'ikamet_ilce', 'adres_2', 'adres_2_posta_kodu',
                   'oda_no', 'oda_tel_no', 'cep_telefonu', 'e_posta', 'e_posta_2', 'e_posta_3',
                   'web_sitesi']

    kaydet = fields.Button(gettext_lazy(u"Kaydet"), cmd="save")
    kps = fields.Button(gettext_lazy(u"KPS'den Adres Bilgileri Getir"), cmd="kps_adres_sorgula")


class DigerBilgilerForm(JsonForm):
    """
    ``KimlikIletisim`` sınıfı için form olarak kullanılacaktır. Form,
    include listesinde, aşağıda tanımlı alanlara sahiptir.

    """

    class Meta:
        include = ['yayinlar', 'projeler', 'verdigi_dersler', 'kan_grubu', 'ehliyet',
                   'unvan', 'biyografi', 'notlar', 'engelli_durumu', 'engel_grubu',
                   'engel_derecesi', 'engel_orani', 'personel_turu']

    kaydet = fields.Button(gettext_lazy(u"Kaydet"), cmd="save")


class KimlikIletisim(CrudView):
    """Kimlik ve İletişim Bilgileri İş Akışı

    Kimlik ve İletişim Bilgileri, aşağıda tanımlı iş akışı adımlarını yürütür:

        - Kimlik Bilgileri Formu
        - Mernis Kimlik Sorgulama
        - Kimlik Bilgileri Kaydet
        - İletişim Bilgileri Formu
        - KPS Adres Sorgulama
        - Iletişim Bilgileri Kaydet
        - Diğer Bilgiler Formu
        - Diğer Bilgileri Kaydet

    Bu iş akışında kullanılan metotlar şu şekildedir:

    Kimlik Bilgilerini Formunu Listele:
       CrudView list metodu kullanılmıştır. Bu metot default olarak tanımlanmıştır.
       Kimlik Bilgileri formunu listeler.

    Mernis'ten Kimlik Bilgileri Getir:
       MERNİS, merkezi nüfus idare sisteminin kısa proje adıdır. Bu metot sayesinde
       personele ait kimlik bilgilerine MERNİS'ten erişilir. Kimlik Bilgileri formundaki
       alanlar MERNİS'ten gelen bilgiler doğrultusunda doldurulur.

    Kaydet:
       MERNİS'ten gelen bilgileri ve yetkili kişinin personelle girdiği bilgileri
       kaydeder. Bu adım ``CrudView.save()`` metodunu kullanır.

    İletişim Bilgilerini Formunu Listele:
       CrudView list metodu kullanılmıştır.Bu metot default olarak tanımlanmıştır.
       İletişim Bilgileri formunu listeler.

    KPS'den Adres Bilgileri Getir:
       Bu metot sayesinde personele ait yerleşim yeri bilgilerine kamu kurumları
       tarafından erişilir. İletişim Bilgileri formundaki alanlar
       KPS'ten gelen bilgiler doğrultusunda doldurulur.

    Kaydet:
       KPS'ten gelen bilgileri ya da yetkili kişinin personelle ilgili girdiği
       bilgileri kaydeder. Bu adım ``CrudView.save()`` metodunu kullanır.

    Diğer Bilgiler Formu:
       CrudView list metodu kullanılmıştır.Bu metot default olarak tanımlanmıştır.
       Diğer Bilgiler formunu listeler.

    Kaydet:
       Yetkili kişinin personelle ilgili girdiği bilgileri kaydeder.
       İş akışı adımdan sonra sona eriyor.

    Bu sınıf ``CrudView`` extend edilerek hazırlanmıştır. Temel model ``Personel``
    modelidir. Meta.model bu amaçla kullanılmıştır.

    Adımlar arası geçiş manuel yürütülmektedir.

    """

    class Meta:
        model = 'Personel'

    def mernis_kimlik_sorgula(self):
        """Mernis Sorgulama

        Zato wrapper metodlarıyla Mernis servisine bağlanır, servisten dönen
        değerlerle nesneyi doldurup kaydeder.

        """

        zs = MernisKimlikBilgileriGetir(tckn=self.object.tckn)
        response = zs.zato_request()
        self.object(**response)
        self.object.save()

    def kps_adres_sorgula(self):
        """KPS Sorgulama

        Zato wrapper metodlarıyla KPS servisine bağlanır, servisten dönen
        değerlerle nesneyi doldurup kaydeder.

        """

        zs = KPSAdresBilgileriGetir(tckn=self.object.tckn)
        response = zs.zato_request()
        self.object(**response)
        self.object.save()

    def kimlik_bilgileri(self):
        """Kimlik Bilgileri Formu"""

        self.form_out(KimlikBilgileriForm(self.object, current=self.current))

    def iletisim_bilgileri(self):
        """İletişim Bilgileri Formu"""

        self.form_out(IletisimBilgileriForm(self.object, current=self.current))

    def diger_bilgiler(self):
        """Diğer Bilgiler Formu"""

        self.form_out(DigerBilgilerForm(self.object, current=self.current))



# -*-  coding: utf-8 -*-
# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
#

"""
    İşten ayrılan personelin modelde arsiv alanı True yapılır.
    Modeldeki notlar alanı ile de işten ayrılan personel ile ilgili açıklama girme
    imkanı verilmiş olur.
"""
from pyoko import ListNode
from zengine.forms import JsonForm
from zengine.forms import fields
from zengine.views.crud import CrudView
from ulakbus.models import Personel, Role
from zengine.lib.translation import gettext as _, gettext_lazy as __
from zengine.models import WFInstance, TaskInvitation
from ulakbus.lib.view_helpers import prepare_choices_for_model


class IstenAyrilmaOnayForm(JsonForm):
    """
        Personel işten ayrılma işleminin onaylanması ve açıklama girilmesi
        amacıyla JsonForm class dan türetilmiş bir classdır.
    """

    class Meta:
        help_text = __(u"Personel İşten ayrılma işlemini onaylıyormusunuz?")
        title = __(u"Personel İşten Ayrılma")
        include = ["notlar"]

    devam_buton = fields.Button(__(u"Onayla"))
    iptal_buton = fields.Button(__(u"İptal"), flow='geri')


class WorkflowAtamaForm(JsonForm):
    class Meta:
        inline_edit = ['yeni_role']

        title = __(u"Kullanıcıya Ait Aktif Görevler")

    bitir_buton = fields.Button(__(u"Bitir"))

    class YeniRoller(ListNode):
        wf_name = fields.String(__(u"WF Adı"), readonly=True)
        eski_role = fields.String(__(u"Eski Rol"), readonly=True)
        yeni_role = fields.String(__(u"Yeni Rol"),
                                  choices=prepare_choices_for_model(Role),
                                  required=True)

    def generate_yeni_roller(self):
        """
            İşten ayrılacak personele ait rol veya roller getirilir. Bu roller de
            aktif iş akışı varmı diye bakılır varsa formda gösterilir.
            Aktif iş akışını atayacağı rolü seçerek, personelin işten ayrılması sağlanır.
        """
        user = Personel.objects.get(self.context.task_data["personel_id"]).user

        for rs in user.role_set:
            wf_instances = [ins.key for ins in WFInstance.objects.filter(
                current_actor=rs.role)]
            queryset = TaskInvitation.objects.filter(progress__in=[20, 30], role=rs.role)

            wf_names = list(set(q.wf_name for q in queryset.filter(instance_id__in=wf_instances)))
            for wf in wf_names:
                self.YeniRoller(
                    wf_name=wf,
                    eski_role=rs.role.name,
                )


class IstenAyrilma(CrudView):
    """
        Personel işten ayrılma wf adımlarının metodlarını içeren
        CrudView dan türetilmiş classdır.
    """

    class Meta:
        model = "Personel"

    def personel_id_kaydet(self):
        # Seçilmiş olan personelin id'si task data da saklanır
        self.current.task_data["personel_id"] = self.current.input["id"]

    def onay_form(self):
        # Onay form kullanıcıya gösteriliyor
        self.form_out(IstenAyrilmaOnayForm(self.object, current=self.current))

    def onayla(self):
        # Onaylanan işten ayrılma veritabanına işleniyor
        personel = Personel.objects.get(self.current.task_data["personel_id"])
        # Burada personelin işten ayrıldığına dair bir açıklama metni girişi yapılmaktadır.
        personel.notlar = self.current.input["form"]["notlar"]
        personel.save()

    def wf_devir(self):
        """
            Aktif gorevlerin baska bir role atanacagi form ekrani
        """
        form = WorkflowAtamaForm(current=self.current)
        form.generate_yeni_roller()
        self.form_out(form)
        self.current.output["meta"]["allow_actions"] = False
        self.current.output["meta"]["allow_add_listnode"] = False

    def wf_devir_onay(self):
        """
            Aktif iş akışlarını yeni seçilen role atama işlemi yapılır.
            İşlem tamamlandıktan sonra ilgili personelin rolü sistemden silinir.
        """
        yeni_roller = self.input['form']['YeniRoller']
        silinecek_roller = []
        if yeni_roller:
            for yr in yeni_roller:
                yeni_rol = Role.objects.get(yr['yeni_role'])
                eski_rol = Role.objects.get(name=yr['eski_role'])
                silinecek_roller.append(eski_rol)
                wf_name = yr['wf_name']
                instances = []

                for wfi in WFInstance.objects.filter(current_actor=eski_rol, name=wf_name):
                    wfi.current_actor = yeni_rol
                    wfi.blocking_save(query_dict={'current_actor': yeni_rol})
                    instances.append(wfi.key)

                for inv in TaskInvitation.objects.filter(progress__in=[20, 30],
                                                         role=eski_rol,
                                                         wf_name=wf_name,
                                                         instance_id__in=instances):
                    inv.role_id = yeni_rol.key
                    inv.blocking_save(query_dict={'role_id': yeni_rol.key})

        for r in silinecek_roller:
            r.blocking_delete()

        personel = Personel.objects.get(self.current.task_data["personel_id"])
        personel.arsiv = True
        personel.save()

    def iptal_islemi(self):
        form = JsonForm(title=_(u'İptal Edildi'))
        form.help_text = _(u"İşlemi iptal ettiniz. Ana sayfa'ya yönlendirileceksiniz")
        form.tamam_buton = fields.Button(_(u"Tamam"))
        self.form_out(form)

    def anasayfa_yonlendirme(self):
        """
            Iptal isleminden sonra ana sayfa ekrani yuklenir.
        """
        self.current.output['cmd'] = 'reload'



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
from collections import defaultdict
from zengine.lib.translation import gettext_lazy, format_currency
from ulakbus.models import *
from ulakbus.views.reports.base import Reporter
from zengine.lib.utils import *


class OgrenciByGender(Reporter):
    TITLE = gettext_lazy(u'Cinsiyete göre öğrenci sayıları')

    def get_objects(self):
        choices = self.convert_choices(Ogrenci().get_choices_for('cinsiyet'))
        result = []
        for val, num in Ogrenci.objects.distinct_values_of('cinsiyet').items():
            try:
                val = int(val)
            except:
                pass
            result.append((choices.get(val, val), num))
        return result


class OgrenciByBrithPlace(Reporter):
    TITLE = gettext_lazy(u'Doğum yerine göre öğrenci sayıları')

    def get_objects(self):
        return [(val, num) for val, num in
                Ogrenci.objects.distinct_values_of('dogum_yeri').items()]


class OgrenciByBrithDate(Reporter):
    TITLE = gettext_lazy(u'Doğum tarihine göre öğrenci sayıları')

    def get_objects(self):
        dates = defaultdict(lambda: 0)
        for val, num in Ogrenci.objects.distinct_values_of('dogum_tarihi').items():
            dates[solr_to_year(val)] += int(num)
        return dates.items()


class OgrenciHarc(Reporter):
    TITLE = gettext_lazy(u'Harç Bilgileri')

    def get_objects(self):
        # choices = self.convert_choices(Borc().get_choices_for('sebep'))
        result = defaultdict(lambda: 0)
        for b in Borc.objects.filter():
            result["%s %s" % (b.get_sebep_display(), 'Borç')] += int(b.miktar or 0)
            # result["%s %s" % (b.get_sebep_display(), 'Ödenen')] += int(b.odenen_miktar or 0)

        return [(k, format_currency(v, 'TRY')) for k, v in result.items()]


class RoomCapacities(Reporter):
    TITLE = gettext_lazy(u'Kapasitesine Göre Mekanlar')

    def get_objects(self):
        return sorted([{'Kapasite': k, 'Oda Sayısı': str(v)} for k, v in
                       Room.objects.distinct_values_of('capacity').items()])
        # return Room.objects.distinct_values_of('capacity').items()



# -*-  coding: utf-8 -*-
# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

from zengine.views.crud import CrudView

class UlakbusView(CrudView):

    def mesaj_kutusu_goster(self, title, msg_type='warning'):
        """
        Hatalı işlem, başarılı işlem gibi bir çok yerde kullanılan mesaj kutularını
        her defasında tanımlamak yerine bu method yardımıyla kullanılmasını sağlar.
        Current task data içerisinde 'msg' olduğunu varsayar ve bu mesajı gösterir.
        Default msgbox type'ı warning olarak belirlenmiştir.

        Args:
            title (string): Mesaj kutusunun başlığı
            type (string): Mesaj kutusunun tipi (warning, info)

        """
        self.current.output['msgbox'] = {
            'type': msg_type, "title": title,
            "msg": self.current.task_data['msg']}
        self.current.task_data['msg'] = None


# -*- coding:utf-8 -*-

from pyoko import Model, field
from zengine.lib.translation import gettext_lazy as _


class HitapSebep(Model):
    sebep_no = field.Integer(_(u"Sebep No"))
    ad = field.String(_(u"Sebep Adı"))
    nevi = field.Integer(_(u"Sebep Nevi"))
    zorunlu_alan = field.String(_(u"Zorunlu Alan"))

    class Meta:
        app = 'Personel'
        verbose_name = _(u"Hitap Sebep Kodu")
        verbose_name_plural = _(u"Hitap Sebep Kodları")
        list_fields = ['sebep_no', 'ad', 'nevi', 'zorunlu_alan']
        search_fields = ['sebep_no', 'ad']

    def __unicode__(self):
        return '%s - %s' % (self.sebep_no, self.ad)



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

import sys
from zengine.management_commands import *

from ..models import Donem, Unit, Sube, Ders, Program, OgrenciProgram, OgrenciDersi, Okutman, Takvim, \
    Building, Room, OgElemaniZamanPlani, ZamanCetveli, DerslikZamanPlani, HAFTA,\
    SinavEtkinligi, OgretimYili
from ulakbus.models import DersEtkinligi, SinavEtkinligi
from common import get_akademik_takvim, SOLVER_MAX_ID, SLOT_SURESI, saat2slot,\
    timedelta2slot, datetime2timestamp
from datetime import datetime, date, timedelta, time
import streamingxmlwriter
import io
import random


def _year():
    return date.today().year


class UnitimeEntityXMLExport(Command):
    EXPORT_DIR = 'bin/dphs/data_exchange/'
    FILE_NAME = ''
    DOC_TYPE = ''
    CMD_NAME = ''
    HELP = ''
    PARAMS = []

    # CPSolver için ID oluştururken pyoko keyleri ile solver idlerini eşleştirmek için kullanılan sözlük
    _SOLVER_IDS = {}

    def _key2id(self, key):
        """Çakışmaları önleyerek pyoko key'lerinden solver id'leri oluşturur."""
        if key in self._SOLVER_IDS:
            return self._SOLVER_IDS[key]
        unitime_key = hash(key) % SOLVER_MAX_ID
        # Çakışmalar çözülene kadar id değerini değiştir
        while unitime_key in self._SOLVER_IDS.values():
            unitime_key += 1
        self._SOLVER_IDS[key] = unitime_key
        return unitime_key

    def _room_id(self, room):
        id_ = room.unitime_key
        if id_ is None:
            id_ = self._key2id(room.key)
            room.unitime_key = id_
            room.save()
        else:
            # Daha sonraki çakışmaları önlemek için, görülen key'i kaydet
            self._SOLVER_IDS[room.key] = id_
        return id_

    def pre_run(self):
        """Export alınmadan önce çağırılır.

        Export'a başlamadan önce yapılması gereken hazırlıklar için
        subclass'lar bu methodu overwrite edebilir.
        """
        pass

    def run(self):
        self.pre_run()
        out_dir = self.create_dir()
        out_file = os.path.join(out_dir, self.FILE_NAME)
        with io.open(out_file, 'wb') as write_stream:
            with streamingxmlwriter.from_stream(write_stream) as writer:
                self.prepare_data(writer)
        print("Veriler %s dizini altinda %s adlı dosyaya kayit edilmiştir" % (out_dir, self.FILE_NAME))

    def write(self, writer):
        """Export alınırken, xml'in basılması için çağırılır.

        Subclass'lar, bu methodu overwrite ederek export etmek istedikleri veriyi burada yazmalıdır.

        Args:
            writer: streamingxmlwriter kütüphanesinden bir writer.
        """
        pass

    def create_dir(self):
        export_directory = self.EXPORT_DIR
        if not os.path.exists(export_directory):
            os.makedirs(export_directory)
        return export_directory

    @property
    def uni(self):
        try:
            return Unit.objects.get(parent_unit_no=0).yoksis_no
        except:
            print("Universite tanimlanmamis")
            sys.exit(1)

    @property
    def term(self):
        try:
            return Donem.objects.get(guncel=True)
        except:
            print("Guncel donem tanimlanmamis")
            sys.exit(1)

    @property
    def bolumler(self):
        b = Unit.objects.filter(unit_type='Bölüm')
        if len(b) > 0:
            return b
        else:
            print("Bolum Bulunamadi")
            sys.exit(1)


class ExportCourseTimetable(UnitimeEntityXMLExport):
    CMD_NAME = 'export_course_timetable'
    HELP = 'Generates all data set Unitime XML import file'
    PARAMS = [{'name': 'bolum', 'type': int, 'required': True,
               'help': 'Bolum olarak yoksis numarasi girilmelidir. Ornek: --bolum 124150'},
                {'name': 'batch_size', 'type': int, 'default': 1000,
               'help': 'Retrieve this amount of records from Solr in one time, defaults to 1000'}]
    FILE_NAME = 'course.xml'
    DOC_TYPE = ''

    # Ders programı modellerindeki günleri, solver'ın günleri ile eşleştiren sözlük
    _GUNLER = {1: '1000000', 2: '0100000', 3: '0010000', 4: '0001000',
               5: '0000100', 6: '0000010', 7: '0000001'}
    # Dersliklerin zaman planlarını hesaplarken kullanılacak minimum zaman, slot sayısı olarak.
    # Bu süreden daha küçük zaman dilimleri planları çıkarırken dikkate alınmayacaktır
    _ODA_ZAMAN_SLOT = 6
    # Room sharing pattern'larında oda kullanımını gösteren semboller
    _ODA_KAPALI = '0'  # Oda herkese kapalı
    _ODA_ACIK = '1'  # Oda herkese açık
    # Bölümlerin value'ları olarak kullanılacak olan karakterler,
    # 'A' ... 'Z' ve 'a' ... 'z' arasındaki tüm ascii karakterlerini içerir
    _BOLUM_KARAKTER = [chr(c) for c in range(65, 91) + range(97, 122)]

    def pre_run(self):
        self.FILE_NAME = '%i.xml' % self.manager.args.bolum

    def prepare_data(self, writer):
        bolum = Unit.objects.get(yoksis_no=self.manager.args.bolum)
        with writer.element('timetable', {
            'version': '2.4',
            'initiative': '%s' % self.uni,
            'term': '%i%s' % (_year(), self.term.ad),
            'created': '%s' % str(date.today()),
            'nrDays': '7',
            'slotsPerDay': '%i' % saat2slot(24)}):
            self.export_rooms(writer)
            self.export_classes(writer, bolum)

    def export_rooms(self, writer):
        with writer.element('rooms'):
            buildings = list(Building.objects.filter())
            for building in buildings:
                rooms = list(Room.objects.filter(building=building))
                for room in rooms:
                    id = self._room_id(room)
                    room_coordinates = '%s,%s' % (room.building.coordinate_x, room.building.coordinate_y)
                    with writer.element('room', {'id': '%i' % id,
                                                 'constraint': 'true',
                                                 'capacity': '%s' % room.capacity,
                                                 'location': room_coordinates,
                                                 }):
                        if room.RoomDepartments:
                            with writer.element('sharing'):
                                bolum_values = {}  # Pattern oluştururken bölümleri valuelar ile eşleştirmek için
                                for j, department in enumerate(room.RoomDepartments):
                                    value = self._BOLUM_KARAKTER[j]
                                    id_ = '%i' % department.unit.yoksis_no
                                    bolum_values[id_] = value
                                    writer.text_element('department', attrs={'value': value, 'id': id_})
                                with writer.element('pattern', {'unit': '%i' % self._ODA_ZAMAN_SLOT}):
                                    writer.characters(self._derslik_zaman_plani(room, bolum_values))
                                writer.text_element('freeForAll', attrs={'value': self._ODA_ACIK})
                                writer.text_element('notAvailable', attrs={'value': self._ODA_KAPALI})

    def _derslik_zaman_plani(self, derslik, bolum_values):
        """Dersliğin zaman planını, solver'a verilecek bir pattern olarak çıkartır.

        Args:
            derslik (Room): Zaman planı çıkarılacak derslik
            bolum_values: Bölüm yoksis no'larını, pattern'da kullanılacak value'lar
                ile eşleştiren sözlük

        Returns:
            str: Solver'a verilecek room sharing pattern'i
        """
        # Haftada ilk gerçekleşenden son gerçekleşene doğru sıralı zaman planları
        zaman_planlari = sorted(DerslikZamanPlani.objects.filter(derslik=derslik),
                                key=lambda p: (p.gun, p.baslangic_saat, p.baslangic_dakika))
        hs, hs_adi = HAFTA[-1]
        hafta_sonu = datetime.min + timedelta(days=hs)
        zaman = datetime.min
        zaman_adimlari = timedelta(minutes=self._ODA_ZAMAN_SLOT*SLOT_SURESI)
        plan = zaman_planlari.pop(0)
        pattern = []
        while zaman < hafta_sonu:
            while self._zaman_araliginda(zaman, plan) > 0:
                try:
                    plan = zaman_planlari.pop(0)
                except IndexError: # Eğer işlenmemiş plan kalmadıysa
                    break
            if self._zaman_araliginda(zaman, plan) == 0:
                if plan.derslik_durum == 1:  # Herkese açık
                    pattern.append(self._ODA_ACIK)
                elif plan.derslik_durum == 2:  # Bölüme ait
                    pattern.append(bolum_values[str(plan.unit.yoksis_no)])
                else:  # Herkese kapalı
                    pattern.append(self._ODA_KAPALI)
            else:
                # Eğer bu zaman için zaman planı girilmediyse odayı herkese kapalı say
                pattern.append(self._ODA_KAPALI)
            zaman = zaman + zaman_adimlari
        return ''.join(pattern)

    @staticmethod
    def _zaman_araliginda(zaman, plan):
        """Verilen zamanın, zaman planının içinde olup olmadığını kontrol eder.

        Args:
            zaman (datetime):
            plan (DerslikZamanPlani):

        Returns:
            int: Eğer zaman, planın içerisinde ise 0, plandan önce ise negatif, plandan
                sonra ise de pozitif bir sayı.
        """
        plan_baslangic = datetime.min + timedelta(days=plan.gun - 1,
                                                  hours=int(plan.baslangic_saat),
                                                  minutes=int(plan.baslangic_dakika))
        plan_bitis = datetime.min + timedelta(days=plan.gun - 1,
                                              hours=int(plan.bitis_saat),
                                              minutes=int(plan.bitis_dakika))
        if zaman < plan_baslangic: return -1
        if plan_bitis <= zaman: return 1
        # Aralıktaysa
        return 0

    def export_classes(self, writer, bolum):
        # Aynı programın aynı dönemindeki ders etkinlikleri arasındaki çakışmaları minimuma indir
        program_sinirlama = {}
        # Aynı şubenin ders etkinliklerine aynı öğrenciler girecektir
        sube_sinirlama = {}
        with writer.element('classes'):
            programlar = list(Program.objects.filter(bolum=bolum))
            for program in programlar:
                donemler_dersler = {}
                for ders in list(Ders.objects.filter(program=program)):
                    donem = ders.program_donemi
                    try:
                        donemler_dersler[donem].append(ders)
                    except KeyError:
                        donemler_dersler[donem] = [ders]
                for donem, donem_dersler in donemler_dersler.items():
                    program_sinirlama[(program.key, donem)] = []
                    for ders in donem_dersler:
                        self._export_ders(writer, bolum, ders, program_sinirlama, sube_sinirlama)
        self._sinirlandirmalar(writer, program_sinirlama, sube_sinirlama)
        writer.text_element('students')

    def _export_ders(self, writer, bolum, ders, program_sinirlama, sube_sinirlama):
        subeler = list(Sube.objects.filter(ders=ders))
        # Önceki exportlardan kalmış olabilecek kayıtları, yenileriyle
        # karışmaması için temizle
        for sube in subeler:
            etkinlikler = DersEtkinligi.objects.filter(bolum=bolum, published=False,
                                                       donem=self.term, sube=sube)
            if etkinlikler.count() > 0:
                etkinlikler.delete()

        derslik_turleri = ders.DerslikTurleri
        ders_turu = [True,False]
        for sube in subeler:
            sube_sinirlama[sube.key] = []
            for i, tur in enumerate(derslik_turleri):
                uygun_derslikler = list(Room.objects.filter(room_type=tur.sinif_turu()))
                sube_subpart_id = '%i' % self._key2id('%i %s' % (i, sube.key))
                okutman = sube.okutman()
                with writer.element('class', {'id': sube_subpart_id,
                                              'offering': '%i' % self._key2id(ders.key),
                                              'config': '%i' % self._key2id(sube.key),
                                              'subpart': sube_subpart_id,
                                              'classLimit': '%i' % sube.kontenjan,
                                              'scheduler': '%i' % bolum.yoksis_no,
                                              'dates': '1111100111110011111001111100', # haftasonları hariç 1 ay her gün
                                              }):
                    program_sinirlama[(ders.program.key, ders.program_donemi)].append(sube_subpart_id)
                    sube_sinirlama[sube.key].append(sube_subpart_id)
                    # Çıkartılan ders için ders etkinliği kaydı oluştur
                    d = DersEtkinligi()
                    d.solved = False
                    d.unitime_key = sube_subpart_id
                    d.unit_yoksis_no = bolum.yoksis_no
                    d.room_type = tur.sinif_turu()
                    d.okutman = okutman
                    d.sube = sube
                    d.donem = self.term
                    d.bolum = bolum
                    d.published = False
                    d.sure = tur.ders_saati
                    d.ek_ders = random.choice(ders_turu)
                    d.save()
                    # Derse uygun derslikleri çıkar
                    for derslik in uygun_derslikler:
                        writer.text_element('room', attrs={'id': '%i' % derslik.unitime_key, 'pref': '0'})

                    writer.text_element('instructor', attrs={'id': '%i' % self._key2id(okutman.key)})
                    self._zamanlari_cikar(writer, okutman, tur, bolum)

    def _zamanlari_cikar(self, writer, ogretim_elemani, tur, bolum):
        """Okutmanın bölüme ait zaman planına göre zaman seçenekleri çıkar

        Args:
            writer: XML çıktısını yazan obje
            okutman (Okutman): Zaman planları kontrol edilecek eğitim görevlisi
            tur (Ders.DerslikTurleri): Zamanların çıkarıldığı dersin ders etkinliği
            bolum (Unit): Uygun zamanları çıkartan bölüm
        """
        # TODO: Eğer öğretim elemanı birden fazla bölümde ders veriyorsa, diğer bölümlerde ders verdiği ders saatleri burada export edilmemeli
        plan = OgElemaniZamanPlani.objects.get(birim=bolum, okutman=ogretim_elemani)
        # Sadece uygun olan zaman cetvelleri
        cetveller = list(ZamanCetveli.objects.filter(birim=bolum, ogretim_elemani_zaman_plani=plan).exclude(durum=3))
        for cetvel in cetveller:
            dilim = cetvel.zaman_dilimi
            zaman = timedelta(hours=int(dilim.baslama_saat), minutes=int(dilim.baslama_dakika))
            bitis = timedelta(hours=int(dilim.bitis_saat), minutes=int(dilim.bitis_dakika))
            gun = self._GUNLER[cetvel.gun]
            sure = int(tur.ders_saati)
            ders_araligi = timedelta(minutes=int(dilim.ders_araligi))
            # Bitiş zamanı gelene kadar, zamanları ekle
            while zaman < bitis:
                # Pazar günü gece yarısını geçen zaman çıkarırsak solver hata veriyor
                if cetvel.gun == 7 and (zaman + timedelta(hours=sure)) >= timedelta(hours=24): break
                writer.text_element('time', attrs={'days': gun,
                                        'start': '%i' % timedelta2slot(zaman),
                                        'length': '%i' % saat2slot(sure),
                                        'breaktime': '%i' % dilim.ara_suresi,
                                        'pref': '%i' % cetvel.durum
                                        })
                zaman += ders_araligi

    def _sinirlandirmalar(self, writer, program_sinirlama, sube_sinirlama):
        with writer.element('groupConstraints'):
            for (program_key, program_donemi), ders_etkinligi_idleri in program_sinirlama.items():
                with writer.element('constraint', {'type': 'SAME_DAYS', 'pref': 'P',
                                                   'id': '%i' % self._key2id('%i %s' % (program_donemi, program_key)),
                                                   }):
                    for etkinlik_id in ders_etkinligi_idleri:
                        writer.text_element('class', attrs={'id': etkinlik_id})
            # TODO: Aynı şubenin derslerini farklı günlere dağıtılması tercih edilmeli
            for sube_key, ders_etkinligi_idleri in sube_sinirlama.items():
                with writer.element('constraint',
                                    {'id': '%i' % self._key2id(sube_key),
                                     'type': 'SAME_STUDENTS', 'pref': 'R',
                                     }):
                    for etkinlik_id in ders_etkinligi_idleri:
                        writer.text_element('class', attrs={'id': etkinlik_id})


class ExportExamTimetable(UnitimeEntityXMLExport):
    """Solver için sınavları export eder.

    Yöksis numarası verilen bölümün sınavlarını, CPSolver ile çözülebilecek bir sınav
    planlama problem olarak export eder.
    """
    CMD_NAME = 'export_exam_timetable'
    HELP = 'Exports a XML file of the exams, to be solved by CPSolver.'
    PARAMS = [{'name': 'bolum', 'type': int, 'required': True,
               'help': 'Bolum olarak yoksis numarasi girilmelidir. Ornek: --bolum 124150'},
              {'name': 'sinav_turleri', 'type': str, 'required': True,
               'help': 'Çıkartılacak sınav türü. '
                       'Virgüllerle ayırarak birden fazla sınav türü seçilebilir. '
                       'Seçenekler için `sinav_turleri`ne bakın.'}]
    FILE_NAME = 'exam.xml'
    DOC_TYPE = ''
    # Sınavların yerleştirilebilecekleri saat aralıkları
    _SINAV_SURELERI = [
        (time(hour=9),  time(hour=12)),
        (time(hour=12), time(hour=14)),
        (time(hour=14), time(hour=17)),
        (time(hour=17), time(hour=20)),
    ]
    # AKADEMIK_TAKVIM_ETKINLIKLERI arasında sınav tarihlerini gösteren etkinliklerin kodları
    _SINAV_ETKINLIKLERI = (26, 58, 65)
    # Bir şubenin sınavı için kullanılacak maksimum oda sayısı
    _SINAV_MAX_ODA = 2
    # Bir şubenin sınavı, birden fazla odada yapılacaksa, her odada olacak minimum öğrenci sayısı
    _SINAV_MIN_KONTENJAN = 10

    def prepare_data(self, writer):
        bolum = Unit.objects.get(yoksis_no=self.manager.args.bolum)
        sinav_turleri = [int(t) for t in self.manager.args.sinav_turleri.split(',')]
        with writer.element('examtt', {'version': '1.0',
                                       'campus': '1',
                                       'term': 'Term',
                                       'year': '%i' % _year(),
                                       'created': str(date.today()),
                                       }):
            # Sınavların gerçekleşebileceği zamanlar
            with writer.element('periods'):
                zamanlar = self._zamanlar(bolum, writer)
            with writer.element('rooms'):
                odalar = self._odalar(bolum, writer)
            with writer.element('students'):
                ogrenci_dersler = self._ogrenciler(bolum, sinav_turleri, writer)
            with writer.element('exams'):
                self._sinavlar(bolum, sinav_turleri, zamanlar, odalar, ogrenci_dersler, writer)
            with writer.element('instructors'):
                self._sinirlandirmalar(bolum, writer)

    def _zamanlar(self, bolum, writer):
        """Sınavların yapılabilecekleri tarih ve saatleri, sınav planı çıktısına ekler.

        Çıkartılan tarih ve saatler, ``period`` xml elemanının ``time`` alanına bakarak
        bulunabilir. Burada, boşluk ile ayrılmış halde başlangıç ve bitiş zamanları,
        saat dilimi bilgisi içermeyen bir POSIX timestamp'i olarak bulunabilir. Bu
        değerler ``datetime.datetime.utcfromtimestamp`` fonksiyonu ile okunabilir.

        Args:
            bolum (Unit): Sınav planı çıkarılacak bölüm.
            writer: XML çıktısını yazacak olan obje
        Returns:
            dict: Sınav sürelerini, bu süre uzunluğundaki zamanlarla eşleştiren bir sözlük.
        """
        tarihler = self._sinav_tarihleri(bolum)
        zaman = tarihler.baslangic
        i = 0
        zamanlar = {}
        while zaman <= tarihler.bitis:
            for j, (sure_bas, sure_bit) in enumerate(self._SINAV_SURELERI):
                baslangic = zaman.replace(hour=sure_bas.hour, minute=sure_bas.minute)
                bitis = zaman.replace(hour=sure_bit.hour, minute=sure_bit.minute)
                zaman = bitis
                id_ = '%i' % (i * len(self._SINAV_SURELERI) + j)
                length = int((bitis - baslangic).seconds / 60)
                writer.text_element('period', attrs={'id': id_,
                                                     'length': '%i' % length,
                                                     'day': '',
                                                     'time': '%f %f' % (datetime2timestamp(baslangic),
                                                                        datetime2timestamp(bitis)),
                                                     'penalty':'0',
                                                     })
                try:
                    zamanlar[length].append(id_)
                except KeyError:
                    zamanlar[length] = [id_]
            zaman = zaman + timedelta(days=1)
            i += 1
        return zamanlar

    def _sinav_tarihleri(self, bolum):
        """Bu bölüm için geçerli olan sınav tarihlerini bulur.

        Args:
            bolum (Unit):

        Returns:
            Takvim:
        """
        yil = OgretimYili.objects.get(yil=Donem.guncel_donem().baslangic_tarihi.year)
        akademik_takvim = get_akademik_takvim(bolum, yil)
        sinav_takvim = None
        for etkinlik in self._SINAV_ETKINLIKLERI:
            try:
                sinav_takvim = Takvim.objects.get(akademik_takvim=akademik_takvim, etkinlik=etkinlik)
            except ObjectDoesNotExist:
                pass
        if not sinav_takvim:
            raise RuntimeError('Sınav etkinliği bulunamadı')
        if not sinav_takvim.baslangic:
            raise RuntimeError('Sınav etkinliğinin başlangıç tarihi tanımlanmamış')
        if not sinav_takvim.bitis:
            raise RuntimeError('Sınav etkinliğinin bitiş tarihi tanımlanmamış')
        return sinav_takvim

    def _odalar(self, bolum, writer):
        """Sınavların yapılabileceği odaları, sınav planı çıktısına ekler.

        Args:
            bolum (Unit): Sınav planı çıkarılacak bölüm.
            writer: XML çıktısını yazacak obje

        Returns:
            `list` of `Room`: Bu bölümün sınav yapabileceği odaların listesi.
        """
        # Bu bölümün ders verebildiği, ve sınav amaçlı kullanılabilen odalar
        odalar = [r for r in Room.objects if bolum in r.RoomDepartments and r.room_type.exam_available == True]
        for oda in odalar:
            writer.text_element('room', attrs={'id': '%i' % self._room_id(oda),
                                               'size': '%i' % oda.capacity,
                                               'alt': '%i' % oda.capacity,
                                               'coordinates': '%s,%s' % (oda.building.coordinate_x,
                                                                         oda.building.coordinate_y),
                                               })
        return odalar

    def _sinav_id(self, sube, sinav):
        return '%i' % self._key2id('%s %i' % (sube.key, sinav.tur))

    def _sinavlar(self, bolum, sinav_turleri, zamanlar, odalar, ogrenci_dersler, writer):
        """Sınavları, sınav planı çıktısına ekler.

        Args:
            bolum (Unit): Sınav planı çıkarılacak bölüm.
            sinav_turleri (`list` of `int`): Plana dahil edilecek sınav türleri.
            zamanlar (dict): Sınav sürelerini, bu süre uzunluğundaki zamanlarla eşleştiren sözlük.
            odalar (`list` of `Room`): Sınavlar için kullanılabilecek olan odalar.
            writer: XML çıktısını yazacak obje
        """
        # TODO: Eğer odalar paylaşılıyorsa, diğer bölümlerle sınav yerlerinin çakışmaması sağlanmalı
        donem = Donem.guncel_donem()
        # Eski exportlardan kalmış olabilecek kayıtları, yenileri ile karışmaması için temizle
        SinavEtkinligi.objects.filter(donem=donem, bolum=bolum, published=False).delete()
        programlar = list(Program.objects.filter(bolum=bolum))
        for program in programlar:
            dersler = list(Ders.objects.filter(program=program, donem=donem))
            for ders in dersler:
                subeler = list(Sube.objects.filter(ders=ders, donem=donem))
                for s, sinav in enumerate(ders.Degerlendirme):
                    if sinav.tur not in sinav_turleri: continue
                    for sube in subeler:
                        sinav_id = self._sinav_id(sube, sinav)
                        with writer.element('exam', {'id': sinav_id,
                                                 'length': '%i' % sinav.sinav_suresi,
                                                 # Alternative seating özelliği, odaların oturma planları için kullanılabilir
                                                 'alt': "false",
                                                 # Her odadaki minimum öğrenci sayısı
                                                 'minSize': '%i' % self._SINAV_MIN_KONTENJAN,
                                                 # Sınavın en çok kaç odaya bölünebileceği
                                                 'maxRooms': '%i' % self._SINAV_MAX_ODA,
                                                 }):
                            etkinlik = SinavEtkinligi(ders=ders, sube=sube, donem=donem, bolum=bolum,
                                           unitime_key=sinav_id, published=False)
                            for ogrenci_id in ogrenci_dersler[ders.key]:
                                etkinlik.Ogrenciler.add(ogrenci_id=ogrenci_id)
                            etkinlik.save()
                            uygun_zamanlar = filter(lambda z: z[0] >= sinav.sinav_suresi, zamanlar.items())
                            for zaman, zaman_idleri in uygun_zamanlar:
                                # Kısa süreli sınavların uzun zaman dilimlerine ayrılmasını önlemek için
                                # farka bağlı olarak bir penalty ata
                                penalty = round((zaman - sinav.sinav_suresi) / 30)
                                for zaman_id in zaman_idleri:
                                    writer.text_element('period', attrs={'id': zaman_id, 'penalty': '%i' % penalty})
                            for oda in odalar:
                                oda_id = '%i' % self._room_id(oda)
                                writer.text_element('room', attrs={'id': oda_id})

    def _ogrenciler(self, bolum, sinav_turleri, writer):
        """Öğrencileri, ve girecekleri sınavları çıktıya ekler.

        Args:
            bolum (Unit): Sınav planı çıkarılacak bölüm.
            sinav_turleri (`list` of `int`): Plana dahil edilecek sınav türleri.
            writer: XML çıktısını yazacak obje
        """
        donem = Donem.guncel_donem()

        dersler = []
        for program in list(Program.objects.filter(bolum=bolum)):
            for ders in list(Ders.objects.filter(program=program, donem=donem)):
                dersler.append(ders)

        # TODO: Eğer öğrenci birden fazla bölümden ders alıyorsa, diğer bölümlerin sınavlarının olduğu period'lar unavailable işaretlenmeli
        ogrenciler = set()
        for ders in dersler:
            # Bu dönem bu dersi alan, devamsızlıktan kalmamış öğrenciler
            ods = {od.ogrenci for od in
                   OgrenciDersi.objects.filter(ders=ders, donem=donem).exclude(katilim_durumu=False)}
            ogrenciler.update(ods)

        ders_ogrenciler = defaultdict(set)
        for ogrenci in ogrenciler:
            with writer.element('student', {'id': '%i' % self._key2id(ogrenci.key)}):
                # Öğrencinin bu dönem aldığı ve devamsızlıktan kalmadığı dersleri
                dersleri = list(OgrenciDersi.objects.filter(ogrenci=ogrenci, donem=donem).exclude(katilim_durumu=False))
                sinav_idleri = set()
                for ders in dersleri:
                    ders_ogrenciler[ders.ders.key].add(ogrenci.key)
                    for sinav in ders.ders.Degerlendirme:
                        if sinav.tur not in sinav_turleri: continue
                        sinav_idleri.add(self._sinav_id(ders.sube, sinav))
                for id_ in sinav_idleri:
                    writer.text_element('exam', attrs={'id': id_})
        return ders_ogrenciler

    def _sinirlandirmalar(self, bolum, constraints):
        pass



#-*- coding: utf-8 -*-

from zato.common import SMTPMessage
import httplib
from ulakbus.services.ulakbus_service import UlakbusService


class EPostaYolla(UlakbusService):

    HAS_CHANNEL = True

    def handle(self):

        default_e_mail = self.request.payload['default_e_mail']
        e_posta =self.request.payload['e_posta']
        message = self.request.payload['message']
        subject = self.request.payload['subject']

        # Obtain a connection
        conn = self.email.smtp.get('Ulakbus-Mail').conn

        # Create a regular e-mail
        msg = SMTPMessage()
        msg.subject = subject
        msg.to = e_posta
        msg.from_ = default_e_mail
        msg.body = message

        # Send the message
        conn.send(msg)

        self.response.status_code = httplib.OK
        self.response.payload = {'status': 'ok', 'result': 'Success'}


# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

from pyoko.modelmeta import model_registry
from pyoko.conf import settings

from ulakbus.views.reports import ReporterRegistry
from zengine.views.base import SysView
from ulakbus.models import Personel, Ogrenci
from zengine.views.menu import Menu


class Search(SysView):
    SEARCH_ON = None
    PATH = None

    def __init__(self, *args, **kwargs):
        super(Search, self).__init__(*args, **kwargs)
        self.query = self.current.input['query']
        self.output['results'] = []
        self.do_search()

    def do_search(self):
        try:
            tckn = int(self.query.strip())
            objects = self.SEARCH_ON.objects.filter(
                tckn__startswith=tckn).values('ad', 'soyad', 'tckn', 'key')
        except ValueError:
            q = self.query.split(" ")
            objects = []
            if len(q) == 1:
                # query Ali, Ayşe, Demir gibi boşluksuz bir string
                # içeriyorsa, ad ve soyad içerisinde aranmalıdır.
                objects = self.SEARCH_ON.objects.search_on(
                    'ad', 'soyad', contains=q[0]).values('ad', 'soyad', 'tckn', 'key')

            if len(q) > 1:
                # query Ali Rıza, Ayşe Han Demir, Neşrin Hasibe Gül Yakuphanoğullarından
                # gibi boşluklu cok parçali bir string ise, öncelikle son parça soyad ile
                # `contains`, önceki parçalar ise isim ile `contains` şeklinde
                # aranmalıdır. Sonuç bulunamaz ise tüm parçalar isim ile
                # `contains` şeklinde aranmalıdır.

                objects = self.SEARCH_ON.objects.search_on(
                    'ad', contains=" ".join(q[:-1])).search_on(
                    'soyad', contains=q[-1]).values('ad', 'soyad', 'tckn', 'key')

                objects_by_name = []
                if not objects:
                    objects = self.SEARCH_ON.objects.search_on(
                        'ad', contains=" ".join(q)).values('ad', 'soyad', 'tckn', 'key')

        self.output['results'] = [("{ad} {soyad}".format(**o), o['tckn'], o['key'], '') for o in
                                  objects]


# @basic_view('ogrenci_ara')
class SearchStudent(Search):
    PATH = 'ogrenci_ara'
    SEARCH_ON = Ogrenci


class SearchPerson(Search):
    PATH = 'personel_ara'
    SEARCH_ON = Personel


class GetCurrentUser(SysView):
    PATH = 'get_current_user'

    def __init__(self, current):
        super(GetCurrentUser, self).__init__(current)
        self.output['current_user'] = {}
        self.getUser(current.user)

    def getUser(self, userObject):
        currentUser = {
            "name": userObject.name,
            "surname": userObject.surname,
            "username": userObject.username,
            "roles": [{"role": role.__unicode__()} for role in userObject.role_set]
        }
        self.output['current_user'] = currentUser


class UlakbusMenu(Menu):
    PATH = 'dashboard'

    def __init__(self, current):
        super(UlakbusMenu, self).__init__(current)
        self.add_reporters()
        self.add_user_data()
        self.add_settings()
        self.add_admin_crud()

    def add_admin_crud(self):
        if self.current.user.superuser:
            for mdl in model_registry.get_base_models():
                self.output['other'].append({
                    "text": mdl.Meta.verbose_name_plural,
                    "wf": 'crud',
                    "model": mdl.__name__,
                    "kategori": 'Admin'})

    def add_settings(self):
        self.output['settings'] = {
            'static_url': settings.S3_PUBLIC_URL,
        }

    def add_user_data(self):
        # add data of current logged in user
        usr = self.current.user
        role = self.current.role
        usr_total_roles = [{"role": roleset.role.__unicode__()} for roleset in
                      self.current.user.role_set]
        self.output['current_user'] = {
            "name": usr.name,
            "surname": usr.surname,
            "username": usr.username,
            "role": self.current.role.abstract_role.name,
            "avatar": usr.get_avatar_url(),
            "is_staff": role.is_staff,
            "is_student": role.is_student,
            "roles": usr_total_roles,
            "role_details": {'unit_name': role.unit.name,
                             'abs_name': role.abstract_role.name,
                             'role_count': len(usr_total_roles)}
        }
        if role.is_student:
            # insert student specific data here
            self.output['current_user'].update({
            })

        elif role.is_staff:
            # insert staff specific data here
            self.output['current_user'].update({
            })

    def add_reporters(self):
        for mdl in ReporterRegistry.get_reporters():
            perm = "report.%s" % mdl['model']
            if self.current.has_permission(perm):
                self.output['other'].append(mdl)



# -*-  coding: utf-8 -*-
# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

from datetime import datetime

from ulakbus.models import Donem
from zengine import forms
from zengine.forms import fields
from zengine.views.crud import CrudView
from zengine.lib.translation import gettext as _, gettext_lazy, format_date


class DonemForm(forms.JsonForm):
    """
    Yeni Dönem Oluşturma iş akışı için kullanılacak formdur.

    """

    guz_baslangic_tarihi = fields.Date(gettext_lazy(u"Başlangıç Tarihi"), format="%d.%m.%Y", required=True)
    guz_bitis_tarihi = fields.Date(gettext_lazy(u"Bitiş Tarihi"), index=True, format="%d.%m.%Y", required=True)
    bahar_baslangic_tarihi = fields.Date(gettext_lazy(u"Başlangıç Tarihi"), index=True, format="%d.%m.%Y",
                                         required=True)
    bahar_bitis_tarihi = fields.Date(gettext_lazy(u"Bitiş Tarihi"), index=True, format="%d.%m.%Y", required=True)

    class Meta:
        grouping = [
            {
                "layout": "4",
                "groups": [
                    {
                        "group_title": gettext_lazy(u"Güz Dönemi"),
                        "items": ['guz_baslangic_tarihi', 'guz_bitis_tarihi'],
                        "collapse": True,
                    }
                ]
            },
            {
                "layout": "4",
                "groups": [
                    {
                        "group_title": gettext_lazy(u"Bahar Dönemi"),
                        "items": ['bahar_baslangic_tarihi', 'bahar_bitis_tarihi'],
                        "collapse": True,

                    }
                ]
            },
            {
                "layout": "7",
                "groups": [
                    {
                        "items": ['kaydet']
                    }
                ]
            }

        ]

    kaydet = fields.Button(gettext_lazy(u"Kaydet"))


class YeniDonemOlusturma(CrudView):
    """
    Yeni Dönem Oluşturma iş akışı üç adımdan oluşmaktadır.

    Dönem Formu Oluştur:
    Güz ve Bahar Dönemi Formu oluşturulur.

    Dönem Formu Kaydet:
    Doldurulan dönem formu bilgilerine göre güz ve bahar dönemi oluşturulup kaydedilir.

    Bilgi Ver:
    İşlemin başarılı olduğuna dair bilgilendirme mesajı basılır.

    """

    class Meta:
        model = "Donem"

    def donem_formu_olustur(self):
        """
        Güz ve Bahar Dönemi Formu oluşturulur.

        """

        _form = DonemForm(current=self.current, title=_(u'Güz ve Bahar Dönemi'))

        son_donem = Donem.son_donem()

        _form.help_text = _(u"""Kayıtlardaki en son donem {donem}
        Başlangıç Tarihi: {baslangic},
        Bitiş Tarihi: {bitis}
        """).format(donem=son_donem.ad,
                    baslangic=format_date(son_donem.baslangic_tarihi),
                    bitis=format_date(son_donem.bitis_tarihi))

        self.form_out(_form)

    def donem_formu_kaydet(self):
        """
        Doldurulan dönem formu bilgilerine göre güz ve bahar dönemi oluşturulup kaydedilir.

        """

        def yeni_donem(ad, baslangic, bitis):
            d = Donem(
                ad='%s - %s' % (ad, baslangic.split('.')[2]),
                baslangic_tarihi=baslangic,
                bitis_tarihi=bitis
            )
            d.save()

        fdata = self.current.input['form']

        yeni_donem(_(u'Bahar'), fdata['bahar_baslangic_tarihi'], fdata['bahar_bitis_tarihi'])
        yeni_donem(_(u'Güz'), fdata['guz_baslangic_tarihi'], fdata['guz_bitis_tarihi'])

    def bilgi_ver(self):
        """
        İşlemin başarılı olduğuna dair bilgilendirme mesajı basılır.

        """

        self.current.output['msgbox'] = {
            'type': 'bilgilendirme', "title": _(u'Güz ve Bahar Dönemi'),
            "msg": _(u'Güz ve Bahar Dönemi başarıyla oluşturulmuştur.')
        }



# -*-  coding: utf-8 -*-
"""Form Modülü

Bu modül `Form` modeli ve bu modelle ilintili data modellerini içerir.

"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

from pyoko import Model, field
from zengine.lib.translation import gettext_lazy as _
from .auth import Role, Permission, User

__author__ = 'H.İbrahim Yılmaz (drlinux)'


class Form(Model):
    """Form Model sınıfı.

    Bu model, genel kullanım amaçlı olarak sisteme yüklenecek olan formların kayıt edileceği data
    modelidir.

    """
    ad = field.String(_(u"Form Adı"), index=True)
    file = field.File(_(u"File"), index=True,
                      random_name=True)  # form eger PDF olarak yulendiyse bu alan kullanilir.
    permissions = Permission()
    date = field.Date(_(u"Form Tarihi"), index=True, format="%d.%m.%Y")

    class Meta:
        app = 'Form'
        verbose_name = _(u"Form")
        verbose_name_plural = _(u"Formlar")
        list_fields = ['ad', 'date']
        search_fields = ['ad', 'file']

    def __unicode__(self):
        return '%s %s' % (self.ad, self.date)


class FormData(Model):
    """FormData Model sınıfı.

    Bu model, `Form` modelinde kayıtlı olan formlara ait dataların tutulacağı data modelidir.
    Veriler data field'ı içine json serialized olarak kayıt edilmektedir.

    """
    form = Form()
    data = field.Text(_(u"Form Data"), index=True)  # form datasi json serialized olarak saklanir
    user = User()
    role = Role()
    date = field.Date(_(u"Form Data Tarihi"), index=True, format="%d.%m.%Y")

    class Meta:
        app = 'Form'
        verbose_name = _(u"Form Data")
        verbose_name_plural = _(u"Form Data")
        list_fields = ['form', 'data', 'date']
        search_fields = ['data', 'date', 'user']

    def _form(self):
        return "%s" % self.form

    def __unicode__(self):
        return '%s %s' % (self.form, self.date)



# -*-  coding: utf-8 -*-
# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
#

"""
    Görevlendirme

    İki tür görevlendirme vardır.
    - Kurum içi görevlendirme
    - Kurum dışı görevlendirme

    Bu iş akışı CrudView nesnesi extend edilerek işletilmektedir.
"""

from zengine.forms import JsonForm
from zengine.forms import fields
from zengine.views.crud import CrudView
from ulakbus.models import Personel, HizmetKayitlari
from ulakbus.models import KurumDisiGorevlendirmeBilgileri, KurumIciGorevlendirmeBilgileri
from zengine.lib.translation import gettext_lazy as __
from ulakbus.lib.role import AbsRole


class GorevlendirmeTurSecForm(JsonForm):
    gorevlendirme_tur = fields.Integer(__(u"Görevlendirme Tür"), choices="gorev_tipi")
    kaydet = fields.Button(__(u"Kaydet ve Devam Et"), cmd="gorevlendirme_tur_kaydet")


class Gorevlendirme(CrudView):
    class Meta:
        model = "Personel"

    def gorevlendirme_tur_sec(self):
        """
            Görevlendirme tipinin seçildiği formu return eden bir metoddur.
        """

        # Seçili personel id si saklanıyor
        self.current.task_data["personel_id"] = self.current.input["id"]
        self.form_out(GorevlendirmeTurSecForm(current=self.current))

    def gorevlendirme_tur_kaydet(self):
        # Görevlendirme türü wf nin ilerleyen adımları için task data da saklandı
        self.current.task_data["gorevlendirme_tur"] = self.current.input["form"][
            "gorevlendirme_tur"]


class KurumIciGorevlendirmeForm(JsonForm):
    class Meta:
        include = ["kurum_ici_gorev_baslama_tarihi", "kurum_ici_gorev_bitis_tarihi",
                   "birim", "soyut_rol", "aciklama",
                   "resmi_yazi_sayi", "resmi_yazi_tarih"]

        title = __(u"Kurum İçi Görevlendirme")

    kaydet = fields.Button(__(u"Kaydet ve Devam Et"), cmd="kaydet")


class KurumIciGorevlendirme(CrudView):
    class Meta:
        model = "KurumIciGorevlendirmeBilgileri"

    def gorevlendirme_form(self):
        self.form_out(KurumIciGorevlendirmeForm(self.object, current=self.current))

    def kaydet(self):
        form_data = self.current.input["form"]
        gorevlendirme = KurumIciGorevlendirmeBilgileri(
            gorev_tipi=self.current.task_data["gorevlendirme_tur"],
            kurum_ici_gorev_baslama_tarihi=form_data["kurum_ici_gorev_baslama_tarihi"],
            kurum_ici_gorev_bitis_tarihi=form_data["kurum_ici_gorev_bitis_tarihi"],
            aciklama=form_data["aciklama"],
            resmi_yazi_sayi=form_data["resmi_yazi_sayi"],
            resmi_yazi_tarih=form_data["resmi_yazi_tarih"],
            birim_id=form_data["birim_id"],
            personel_id=self.current.task_data["personel_id"],
            soyut_rol_id=form_data["soyut_rol_id"]
        )

        gorevlendirme.blocking_save()

        self.current.task_data["hizmet_cetvel_giris"] = form_data["soyut_rol_id"] in [
            AbsRole.FAKULTE_DEKANI.name, AbsRole.REKTOR.name]


class KurumDisiGorevlendirmeForm(JsonForm):
    class Meta:
        include = ["kurum_disi_gorev_baslama_tarihi", "kurum_disi_gorev_bitis_tarihi", "aciklama",
                   "resmi_yazi_sayi", "resmi_yazi_tarih", "maas", "yevmiye", "yolluk", "ulke",
                   "soyut_rol"]
        title = __(u"Kurum Dışı Görevlendirme")

    kaydet = fields.Button(__(u"Kaydet ve Devam Et"), cmd="kaydet")


class KurumDisiGorevlendirme(CrudView):
    class Meta:
        model = "KurumDisiGorevlendirmeBilgileri"

    def gorevlendirme_form(self):
        self.form_out(KurumDisiGorevlendirmeForm(self.object, current=self.current))

    def kaydet(self):
        form_data = self.current.input["form"]
        gorevlendirme = KurumDisiGorevlendirmeBilgileri(
            kurum_disi_gorev_baslama_tarihi=form_data["kurum_disi_gorev_baslama_tarihi"],
            kurum_disi_gorev_bitis_tarihi=form_data["kurum_disi_gorev_bitis_tarihi"],
            aciklama=form_data["aciklama"],
            resmi_yazi_sayi=form_data["resmi_yazi_sayi"],
            resmi_yazi_tarih=form_data["resmi_yazi_tarih"],
            maas=form_data["maas"],
            yevmiye=form_data["yevmiye"],
            yolluk=form_data["yolluk"],
            ulke=form_data["ulke"],
            soyut_rol_id=form_data["soyut_rol_id"],
            personel_id=self.current.task_data["personel_id"]
        )
        gorevlendirme.blocking_save()

        self.current.task_data["hizmet_cetvel_giris"] = form_data["soyut_rol_id"] in [
            AbsRole.FAKULTE_DEKANI.name, AbsRole.REKTOR.name]


class HizmetCetveliForm(JsonForm):
    class Meta:
        include = ["kayit_no", "baslama_tarihi", "bitis_tarihi", "gorev", "unvan_kod", "yevmiye",
                   "ucret", "hizmet_sinifi",
                   "kadro_derece", "odeme_derece", "odeme_kademe", "odeme_ekgosterge",
                   "kazanilmis_hak_ayligi_derece",
                   "kazanilmis_hak_ayligi_kademe", "kazanilmis_hak_ayligi_ekgosterge",
                   "emekli_derece", "emekli_kademe",
                   "emekli_ekgosterge", "sebep_kod", "kurum_onay_tarihi"]

    kaydet_buton = fields.Button(__(u"Kaydet"))


class HizmetCetveli(CrudView):
    class Meta:
        model = "HizmetKayitlari"

    def giris_form(self):
        self.form_out(HizmetCetveliForm(self.object, current=self.current))

    def kaydet(self):
        form_data = self.current.input["form"]
        personel = Personel.objects.get(self.current.task_data["personel_id"])
        hizmet_kayitlari = HizmetKayitlari(
            personel_id=personel.key,
            tckn=personel.tckn,
            baslama_tarihi=form_data["baslama_tarihi"],
            bitis_tarihi=form_data["bitis_tarihi"],
            gorev=form_data["gorev"],
            unvan_kod=form_data["unvan_kod"],
            yevmiye=form_data["yevmiye"],
            ucret=form_data["ucret"],
            hizmet_sinifi=form_data["hizmet_sinifi"],
            kadro_derece=form_data["kadro_derece"],
            odeme_derece=form_data["odeme_derece"],
            odeme_kademe=form_data["odeme_kademe"],
            odeme_ekgosterge=form_data["odeme_ekgosterge"],
            kazanilmis_hak_ayligi_derece=form_data["kazanilmis_hak_ayligi_derece"],
            kazanilmis_hak_ayligi_kademe=form_data["kazanilmis_hak_ayligi_kademe"],
            kazanilmis_hak_ayligi_ekgosterge=form_data["kazanilmis_hak_ayligi_ekgosterge"],
            emekli_derece=form_data["emekli_derece"],
            emekli_kademe=form_data["emekli_kademe"],
            emekli_ekgosterge=form_data["emekli_ekgosterge"],
            kurum_onay_tarihi=form_data["kurum_onay_tarihi"]
        )
        if "sebep_kod" in form_data:
            hizmet_kayitlari.sebep_kod = form_data["sebep_kod"]
        hizmet_kayitlari.blocking_save()



# -*-  coding: utf-8 -*-
# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

from ulakbus.models import SinavEtkinligi, Room, Donem
from zengine.forms import JsonForm
from zengine.forms import fields
from zengine.views.crud import CrudView
from collections import OrderedDict
from ulakbus.lib.date_time_helper import map_etkinlik_hafta_gunleri, HAFTA
from zengine.lib.translation import gettext as _, gettext_lazy


class DerslikSecimFormu(JsonForm):
    """
    Derslik Sınav ProgramLaı iş akışının sınav seç adımında kullanılan form.
    """

    ileri = fields.Button(gettext_lazy(u"İleri"))


class DerslikSinavProgramlari(CrudView):
    """ Derslik Sınav Programları
    Derslik Sınav Programları iş akışı, seçilen dersliğe ait sınav programlarını gösterip kullanıcının
    yazdırabilmesine olanak sağlar.

    İş akışı 4 adımdan oluşur.

    Sınav Etkinliklerini Kontrol Et:
    Yayınlanmış sınav etkinlikleri var mı yok mu diye kontrol eder.

    Bilgi Ver:
    Yayınlananmış sınavlar yok ise bilgi mesajı  ekrana basılır.

    Derslik Seç:
    Derslik Seçilir.

    Derslik Sınav Programlarını Göster:
    Seçilen dersliğe ait sınavlar etkinliklerinin ders adını ve tarihi gösterir.

    """

    class Meta:
        model = "SinavEtkinligi"

    def sinav_etkinliklerini_kontrol_et(self):
        """
        Yayınlanmış sınav etkinlikleri var mı yok mu diye kontrol eder.

        """
        sinav_etkinlikleri = SinavEtkinligi.objects.filter(published=True,
                                                           donem=Donem.guncel_donem(self.current),
                                                           bolum=self.current.role.unit)
        self.current.task_data['yayinlanmamis_sinav_sayisi'] = len(sinav_etkinlikleri)

    def bilgi_ver(self):
        """
        Yayınlananmış sınavlar yok ise bilgi mesajı  ekrana basılır.

        """
        self.current.output['msgbox'] = {
            'type': 'info', "title": _(u'Yayınlanmamış Sınavlar'),
            'msg': _(u"Yayınlanmış sınavlar bulunmamaktadır.")
        }

    def derslik_sec(self):
        """
        Derslik Seçilir.

        """
        _form = DerslikSecimFormu(title=_(u"Derslik Seçiniz"), current=self.current)
        _choices = []
        for s_etkinlik in SinavEtkinligi.objects.filter(published=True,
                                                        donem=Donem.guncel_donem(self.current),
                                                        bolum=self.current.role.unit):
            for s_yerleri in s_etkinlik.SinavYerleri:
                if s_etkinlik.SinavYerleri:
                    _choices.append((s_yerleri.room.key, s_yerleri.room.__unicode__()))
        _form.derslik = fields.Integer(choices=_choices)
        self.form_out(_form)

    def derslik_sinav_programlarini_goster(self):
        """
        Seçilen dersliğe ait sınavlar etkinliklerinin ders adını ve tarihi gösterir.

        """

        room = Room.objects.get(self.current.input['form']['derslik'])
        s_etkinlikleri = [s for s in SinavEtkinligi.objects.order_by('tarih') if
                          room in s.SinavYerleri]
        sinav_etkinlikleri = map_etkinlik_hafta_gunleri(s_etkinlikleri)
        hafta = dict(HAFTA)
        self.output['objects'] = [hafta]

        # Bir güne ait maximum etkinlik sayısı.
        max_etkinlik = max(map(len, sinav_etkinlikleri.values()))
        for i in range(max_etkinlik):
            sinav_etkinlik_list = OrderedDict({})
            for hafta_gun in hafta.keys():
                if hafta_gun in sinav_etkinlikleri:
                    try:
                        etkinlik = sinav_etkinlikleri[hafta_gun][i]
                        sinav_etkinlik_list[hafta[hafta_gun]] = etkinlik
                    except IndexError:
                        sinav_etkinlik_list[hafta[hafta_gun]] = ''

                else:
                    sinav_etkinlik_list[hafta[hafta_gun]] = ''
            item = {
                "type": "table-multiRow",
                "fields": sinav_etkinlik_list,
                "actions": False,
                'key': ''
            }
            self.output['objects'].append(item)
        _form = JsonForm(title=_(u"%s Dersliğine Ait Sınav Programlar") % room.__unicode__())
        _form.yazdir = fields.Button(_(u"Pdf Yazdır"))
        self.form_out(_form)
        self.current.output["meta"]["allow_actions"] = False
